from .drawwidget import AdwDrawWidget


class AdwButton(AdwDrawWidget):
    def __init__(self,
                 *args,
                 command=None,
                 drawmode: int = 1,
                 text: str = "",
                 radius: int = 14,
                 **kwargs):
        super().__init__(*args, **kwargs)

        if command is None:
            def _():
                pass
            command = _

        # 初始属性
        self.args(
            drawmode=drawmode,
            back_default="#fdfdfd",
            back_enter="#f9f9f9",
            back_button="#fafafa",
            border_default="#ededed",
            border_enter="#d5d5d5",
            border_button="#ebebeb",
            border_width_default=1,
            border_width_enter=1,
            border_width_button=1,
            radius=radius,
            text=text,
            text_default="#202020",
            text_enter="#202020",
            text_button="#202020",
        )

        self.bind("<<Click>>", lambda event=None: command())

        self.default_palette()

    def _draw(self, event=None):
        super()._draw(event)
        if self._is_enter:
            if self._is_button:
                __back = self._back_button
                __border = self._border_button
                __border_width = self._border_width_button
                __text = self._text_button
            else:
                __back = self._back_enter
                __border = self._border_enter
                __border_width = self._border_width_enter
                __text = self._text_enter
        else:
            __back = self._back_default
            __border = self._border_default
            __border_width = self._border_width_default
            __text = self._text_default

        # 绘制框架
        if self._drawmode == 0:
            self.roundrect_draw(
                x=0, y=0,
                width=self.winfo_width(), height=self.winfo_height(),
                fill=__border, outline=__border, radius=self._radius + 2, tag="frame_border"
            )
            self._frame_border = "frame_border"
            self.roundrect_draw(
                x=__border_width, y=__border_width,
                width=self.winfo_width() - 2 * __border_width,
                height=self.winfo_height() - 2 * __border_width,
                fill=__back, outline=__back, radius=self._radius, tag="frame"
            )
        elif self._drawmode == 1:
            self.roundrect2_draw(
                x1=0, y1=0,
                x2=self.winfo_width() - __border_width,
                y2=self.winfo_height() - __border_width,
                fill=__back, outline=__border, radius=self._radius, tag="frame"
            )
        self._frame = "frame"

        # 绘制文字
        self._label = self.create_text(
            self.winfo_width() / 2, self.winfo_height() / 2,
            text=self._text, fill=__text
        )

    def command(self, func):
        self.bind("<<Click>>", lambda event=None: func())

    def default_palette(self):
        pass

    def palette(self, palette: dict):
        if "button" in palette:
            if "radius" in palette["button"]:
                self._radius = palette["button"]["radius"]
            if "default" in palette["button"]:
                if "back" in palette["button"]["default"]:
                    self._back_default = palette["button"]["default"]["back"]
                if "border" in palette["button"]["default"]:
                    self._border_default = palette["button"]["default"]["border"]
                if "border_width" in palette["button"]["default"]:
                    self._border_width_default = palette["button"]["default"]["border_width"]
                if "fore" in palette["button"]["default"]:
                    self._text_default = palette["button"]["default"]["fore"]
            if "active" in palette["button"]:
                if "back" in palette["button"]["active"]:
                    self._back_enter = palette["button"]["active"]["back"]
                if "border" in palette["button"]["active"]:
                    self._border_enter = palette["button"]["active"]["border"]
                if "border_width" in palette["button"]["active"]:
                    self._border_width_enter = palette["button"]["active"]["border_width"]
                if "fore" in palette["button"]["active"]:
                    self._text_enter = palette["button"]["active"]["fore"]
            if "pressed" in palette["button"]:
                if "back" in palette["button"]["pressed"]:
                    self._back_button = palette["button"]["pressed"]["back"]
                if "border" in palette["button"]["pressed"]:
                    self._border_button = palette["button"]["pressed"]["border"]
                if "border_width" in palette["button"]["pressed"]:
                    self._border_width_button = palette["button"]["pressed"]["border_width"]
                if "fore" in palette["button"]["pressed"]:
                    self._text_button = palette["button"]["pressed"]["fore"]
        self.update()



from tkinter.font import Font, nametofont
from tkadwold.windows.canvas.widget import AdwWidget


# Text
class AdwDrawBasicText(AdwWidget):
    def __init__(self, *args, text: str = "", width=120, height=80, **kwargs):

        super().__init__(*args, width=width, height=height, highlightthickness=0, **kwargs)

        from tkinter import Text

        self.text_text_font = nametofont("TkDefaultFont")

        self.text = Text(self, bd=0, font=self.text_text_font, undo=True, highlightthickness=0)

        self.text.insert("1.0", text)

        self._other()

        self.default_palette()

        self._text_back = self.text_back
        self._text_border = self.text_border
        self._text_text_back = self.text_text_back
        self._text_bottom_line = self.text_bottom_line
        self._text_bottom_width = self.text_bottom_width

        self.bind("<Configure>", self._draw, add="+")
        self.bind("<Button>", self._click, add="+")
        self.bind("<Enter>", self._hover, add="+")
        self.bind("<Leave>", self._hover_release, add="+")
        #self.bind("<FocusIn>", self._focus, add="+")
        self.text.bind("<FocusIn>", self._focus, add="+")
        #self.bind("<FocusOut>", self._focusout, add="+")
        self.text.bind("<FocusOut>", self._focusout, add="+")

        self.bind("<Control-z>", lambda event: self.undo(), add="+")
        self.bind("<Control-Z>", lambda event: self.undo(), add="+")
        self.bind("<Control-Shift-z>", lambda event: self.redo(), add="+")
        self.bind("<Control-Shift-Z>", lambda event: self.redo(), add="+")

        self.text.bind("<Control-z>", lambda event: self.undo(), add="+")
        self.text.bind("<Control-Z>", lambda event: self.undo(), add="+")
        self.text.bind("<Control-Shift-z>", lambda event: self.redo(), add="+")
        self.text.bind("<Control-Shift-Z>", lambda event: self.redo(), add="+")

        self._draw(None)

    def tbbox(self, *args, **kwargs):
        return self.text.bbox(*args, **kwargs)

    def compare(self, *args, **kwargs):
        return self.text.compare(*args, **kwargs)

    def count(self, *args, **kwargs):
        return self.text.count(*args, **kwargs)

    def debug(self, *args, **kwargs):
        return self.text.debug(*args, **kwargs)

    def tdelete(self, *args, **kwargs):
        return self.text.delete(*args, **kwargs)

    def dump(self, *args, **kwargs):
        return self.text.dump(*args, **kwargs)

    def get(self, *args, **kwargs):
        return self.text.get(*args, **kwargs)

    def tindex(self, *args, **kwargs):
        return self.text.index(*args, **kwargs)

    def tinsert(self, *args, **kwargs):
        return self.text.insert(*args, **kwargs)

    def search(self, *args, **kwargs):
        return self.text.search(*args, **kwargs)

    def see(self, *args, **kwargs):
        return self.text.see(*args, **kwargs)

    def undo(self):
        try:
            self.text.edit_undo()
        except:
            pass

    def redo(self):
        try:
            self.text.edit_redo()
        except:
            pass

    def _other(self):
        if hasattr(self, "button_frame_back"):
            self.configure(background=self.button_frame_back, borderwidth=0)

    def _draw(self, evt):
        self.delete("all")

        self.text_frame = self.create_rectangle(
            0, 0, self.winfo_width() - 1, self.winfo_height() - 1,
            width=self.text_border_width,
            outline=self._text_border, fill=self._text_back,
        )

        self.text_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self.text_border_width - 5 - self.text_padding[0],
            height=self.winfo_height() - self.text_border_width - 5 - self.text_padding[1],
            window=self.text
        )

        self.text_bottom = self.create_rectangle(1, self.winfo_height() - self._text_bottom_width - 1,
                                                 self.winfo_width() - 1, self.winfo_height() - 1,
                                                 fill=self._text_bottom_line, outline=self._text_bottom_line,
                                                 width=0)

        if self._text_bottom_width == 0:
            self.delete(self.text_bottom)

        self.tag_raise(self.text_bottom, self.text_text)

        self.text.configure(background=self._text_back, foreground=self._text_text_back, autoseparators=True, insertbackground=self._text_text_back)

    def _focus(self, evt=None):
        self._text_back = self.text_focusin_back
        self._text_border = self.text_focusin_border
        self._text_text_back = self.text_focusin_text_back
        self._text_bottom_line = self.text_focusin_bottom_line
        self._text_bottom_width = self.text_focusin_bottom_width

        self._draw(None)

    def _focusout(self, evt=None):
        self._text_back = self.text_back
        self._text_border = self.text_border
        self._text_text_back = self.text_text_back
        self._text_bottom_line = self.text_bottom_line
        self._text_bottom_width = self.text_bottom_width

        self._draw(None)

    def _click(self, evt=None):
        self.focus_set()

    def _hover(self, evt=None):
        self.hover = True

    def _hover_release(self, evt=None):
        if not self.focus_get():
            self.hover = False
            self._text_back = self.text_back
            self._text_border = self.text_border
            self._text_text_back = self.text_text_back
            self._text_bottom_line = self.text_bottom_line
            self._text_bottom_width = self.text_bottom_width

            self._draw(None)

    def font(self, font: Font = None):
        if font is None:
            return self.text_text_font
        else:
            self.text_text_font = font
        self.update()

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "text": {
                    "padding": (3, 4),

                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#5f5f5f",
                    "border_width": 1,

                    "bottom_line": "#eaeaea",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#1a1a1a",
                        "border_width": 1,

                        "bottom_line": "#185fb4",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "text": {
                    "padding": (3, 4),

                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#cecece",
                    "border_width": 1,

                    "bottom_line": "#ffffff",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#ffffff",
                        "border_width": 1,

                        "bottom_line": "#4cc2ff",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "text" in dict:
                self.text_padding = dict["text"]["padding"]

                self.text_back = dict["text"]["back"]
                self.text_border = dict["text"]["border"]
                self.text_text_back = dict["text"]["text_back"]
                self.text_border_width = dict["text"]["border_width"]

                self.text_bottom_line = dict["text"]["bottom_line"]
                self.text_bottom_width = dict["text"]["bottom_width"]

                if "focusin" in dict["text"]:
                    self.text_focusin_back = dict["text"]["focusin"]["back"]
                    self.text_focusin_border = dict["text"]["focusin"]["border"]
                    self.text_focusin_text_back = dict["text"]["focusin"]["text_back"]
                    self.text_focusin_border_width = dict["text"]["focusin"]["border_width"]

                    self.text_focusin_bottom_line = dict["text"]["focusin"]["bottom_line"]
                    self.text_focusin_bottom_width = dict["text"]["focusin"]["bottom_width"]

            self._text_back = self.text_back
            self._text_border = self.text_border
            self._text_text_back = self.text_text_back
            self._text_bottom_line = self.text_bottom_line
            self._text_bottom_width = self.text_bottom_width

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwDrawText(AdwDrawBasicText):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkText(AdwDrawBasicText):
    def default_palette(self):
        self.palette_dark()


# Rounded text
class AdwDrawBasicRoundText(AdwDrawBasicText):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _other(self):
        self.configure(background=self.master.cget("bg"), borderwidth=0)

    def border_radius(self, radius=None):
        if radius is None:
            return self.button_radius
        else:
            self.button_radius = radius

    def _draw(self, evt):
        self.delete("all")

        self.text_frame = self.create_round_rect2(
            2, 2, self.winfo_width() - 3, self.winfo_height() - 3, self.text_radius,
            width=self.text_border_width,
            outline=self._text_border, fill=self._text_back,
        )

        self.text_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self.text_border_width - 5 - self.text_padding[0],
            height=self.winfo_height() - self.text_border_width - 5 - self.text_padding[1],
            window=self.text
        )

        self.text_bottom = self.create_rectangle(3 + self.text_radius / 2,
                                                 self.winfo_height() - self._text_bottom_width - 3,
                                                 self.winfo_width() - 3 - self.text_radius / 2,
                                                 self.winfo_height() - 3.5,
                                                 fill=self._text_bottom_line, outline=self._text_bottom_line,
                                                 width=0)

        if self._text_bottom_width == 0:
            self.delete(self.text_bottom)

        self.tag_raise(self.text_bottom, self.text_text)

        self.text.configure(background=self._text_back, foreground=self._text_text_back, insertbackground=self._text_text_back)

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "text": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#5f5f5f",
                    "border_width": 1,

                    "bottom_line": "#eaeaea",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#1a1a1a",
                        "border_width": 1,

                        "bottom_line": "#185fb4",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "text": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#cecece",
                    "border_width": 1,

                    "bottom_line": "#ffffff",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#ffffff",
                        "border_width": 1,

                        "bottom_line": "#4cc2ff",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if "text" in dict:
            self.text_radius = dict["text"]["radius"]


class AdwDrawRoundText(AdwDrawBasicRoundText):
    def default_palette(self):
        self.palette_light()


class AdwDrawRoundDarkText(AdwDrawBasicRoundText):
    def default_palette(self):
        self.palette_dark()


class AdwDrawBasicRoundText3(AdwDrawBasicRoundText):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _other(self):
        self.configure(background=self.master.cget("bg"), borderwidth=0)

    def border_radius(self, radius=None):
        if radius is None:
            return self.button_radius
        else:
            self.button_radius = radius

    def _draw(self, evt):
        self.delete("all")

        self.create_round_rect4(
            0,
            0,
            self.winfo_width() - 1,
            self.winfo_height() - 1,
            self.text_radius,
            width=self.text_border_width,
            outline=self._text_border, fill=self._text_back,
        )

        self.text_frame = "button_frame"

        self.text_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self.text_border_width - self.text_padding[0] - 2,
            height=self.winfo_height() - self.text_border_width - self.text_padding[1] - 2,
            window=self.text
        )

        self.text_bottom = self.create_rectangle(1 + self.text_radius / 5,
                                                 self.winfo_height() - self._text_bottom_width - 1,
                                                 self.winfo_width() - 1 - self.text_radius / 5,
                                                 self.winfo_height() - 1,
                                                 fill=self._text_bottom_line, outline=self._text_bottom_line,
                                                 width=0)

        if self._text_bottom_width == 0:
            self.delete(self.text_bottom)

        self.tag_raise(self.text_bottom, self.text_text)

        self.text.configure(background=self._text_back, foreground=self._text_text_back, insertbackground=self._text_text_back)

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "text": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#5f5f5f",
                    "border_width": 1,

                    "bottom_line": "#eaeaea",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#1a1a1a",
                        "border_width": 1,

                        "bottom_line": "#185fb4",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "text": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#cecece",
                    "border_width": 1,

                    "bottom_line": "#ffffff",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#ffffff",
                        "border_width": 1,

                        "bottom_line": "#4cc2ff",
                        "bottom_width": 2,
                    }
                },
            }
        )


class AdwDrawRoundText3(AdwDrawBasicRoundText3):
    def default_palette(self):
        self.palette_light()


class AdwDrawRoundDarkText3(AdwDrawBasicRoundText3):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkinter import Tk

    root = Tk()

    text1 = AdwDrawText()
    text1.pack(fill="x", padx=5, pady=5)

    text2 = AdwDrawDarkText()
    text2.pack(fill="x", padx=5, pady=5)

    text3 = AdwDrawRoundText()
    text3.pack(fill="x", padx=5, pady=5)

    text4 = AdwDrawRoundDarkText()
    text4.pack(fill="x", padx=5, pady=5)

    text5 = AdwDrawRoundText3()
    text5.pack(fill="x", padx=5, pady=5)

    text6 = AdwDrawRoundDarkText3()
    text6.pack(fill="x", padx=5, pady=5)

    root.mainloop()



class Arg(object):
    def __init__(self, master, arg, kwargs):
        self.master = master
        self.arg = arg
        self.kwargs = kwargs

        if not hasattr(self.master, "_"+arg):
            setattr(self.master, "_" + arg, kwargs[arg])
            setattr(self.master, "set_" + arg, lambda value: setattr(self.master, "_"+arg, value))
            setattr(self.master, "get_" + arg, lambda: getattr(self.master, "_"+arg))


class AdwBase():
    def args(self, **kwargs):
        """
        初始或设置多个属性
        """
        if kwargs:
            for arg in kwargs:
                setattr(self, "_arg_"+arg, Arg(self, arg, kwargs))

            self.update()
        else:
            return getattr(self, name)




from tkadwold.windows.canvas.frame import AdwDrawRoundFrame3
from tkadwold.windows.canvas.button import AdwDrawCircularButton



class AdwMDI(AdwDrawRoundFrame3):
    """
    多文档窗口
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.mdichildren = []

    def create_child(self, x=10, y=10, width=150, height=150, closebutton=AdwDrawCircularButton, can_close: bool = True,
                     childframe=AdwDrawRoundFrame3, titlebarframe=AdwDrawRoundFrame3, content=AdwDrawRoundFrame3):
        """
        创建一个子窗口

        :return: AdwTFrame
        """
        child = childframe(self.frame)
        child.place(x=x, y=y, width=width, height=height)

        child.titlebar = titlebarframe(child.frame, height=34)
        child.titlebar.frame_border_width = 0
        child.titlebar.frame_border = child.frame_back
        child.closebutton = closebutton(child.titlebar.frame, text="✕", width=25, height=25)
        if can_close:  # 不推荐使用
            child.closebutton.configure(command=lambda: child.place_forget())
        child.closebutton.pack(anchor="ne", padx=2, pady=2)
        child.titlebar.row(fill="x", side="top", padx=2, pady=2)

        child.titlebar.frame.bind("<Button-1>", self._click, add="+")
        child.titlebar.frame.bind("<B1-Motion>", lambda event: self._move(event, child), add="+")

        child.content = content(child.frame, height=120)
        child.content.row(fill="both", expand="yes", padx=2, pady=2)

        self.mdichildren.append(child)

        return child.content, child

    def create_designer_child(self, x=10, y=10, width=150, height=150, closebutton=AdwDrawCircularButton, can_close: bool = True,
                              childframe=AdwDrawRoundFrame3, titlebarframe=AdwDrawRoundFrame3, content=AdwDrawRoundFrame3):
        from tkadwold.designer.designerframe import AdwDesignerFrame
        child = AdwDesignerFrame(self, childframe)
        child.place(x=x, y=y, width=width, height=height)

        child.titlebar = titlebarframe(child.widget, height=34)
        child.titlebar.frame_border_width = 0
        child.titlebar.frame_border = child.widget.frame_back
        child.closebutton = closebutton(child.titlebar.frame, text="✕", width=25, height=25)
        if can_close:  # 不推荐使用
            child.closebutton.configure(command=lambda: child.place_forget())
        child.closebutton.pack(anchor="ne", padx=2, pady=2)
        child.titlebar.pack(fill="x", side="top", padx=2, pady=2)

        child.titlebar.frame.bind("<Button-1>", self._click, add="+")
        child.titlebar.frame.bind("<Button-1>", lambda event: child.titlebar.frame.focus_set(), add="+")
        child.titlebar.frame.bind("<B1-Motion>", lambda event: self._move(event, child), add="+")

        child.content = content(child.widget, height=120)
        child.content.row(fill="both", expand="yes", padx=6, pady=6)

        self.mdichildren.append(child)

        return child.content, child

    def _click(self, event):
        self.x, self.y = event.x, event.y

    def _move(self, event, child):
        child.place(
            x=(event.x - self.x) + child.winfo_x(),
            y=(event.y - self.y) + child.winfo_y()
        )

    def palette(self, dict=None):
        super().palette(dict)
        if hasattr(self, "mdichildren"):
            for child in self.mdichildren:
                child.palette(dict)


if __name__ == '__main__':
    from tkadwold import Adwite, set_default_theme, AdwTCircularButton, AdwTFrame, AdwTDesignerFrame

    set_default_theme("gtk", "light")

    root = Adwite()

    mdi = AdwMDI()
    mdiChild1 = mdi.create_child(childframe=AdwTFrame, titlebarframe=AdwTFrame, closebutton=AdwTCircularButton)
    mdiChild2 = mdi.create_designer_child(childframe=AdwTFrame, titlebarframe=AdwTFrame, closebutton=AdwTCircularButton)
    mdi.pack(fill="both", expand="yes", padx=10, pady=10)

    root.mainloop()



from functools import partial

from .drawengine import AdwDrawEngine
from tkinter import Widget, Frame


class AdwDesignerFrame(AdwDrawEngine):
    def __init__(self, master=None, widget: Widget = Frame,
                 candragged: bool = True,
                 *args, **kwargs):
        """
        代码修改自 https://blog.csdn.net/tinga_kilin/article/details/113773393
        """
        super().__init__(master=master)

        self.default_palette()

        self.configure(bd=0, highlightthickness=0)

        try:
            self.configure(background=self.master.cget("background"))
            if hasattr(self.master, "frame_back"):
                self.configure(background=self.master.frame_back)
            self.update()
        except:
            pass

        self.widget = widget(self, *args, **kwargs)
        self.widget.pack(fill='both', expand=True, pady=9, padx=9)

        self.widget.bind("<Button-1>", self.mouse_down, add='+')
        self.candragged(candragged)
        self.widget.bind('<FocusOut>', lambda event: self.delete('all'))
        self.widget.bind('<FocusIn>', lambda event: (self._draw(), self.show()))

        self.is_sizing = False
        self.old_width = 0
        self.old_height = 0
        self.old_pos_x = 0
        self.old_pos_y = 0
        self.start_x = 0
        self.start_y = 0
        self.start_root_x = 0
        self.start_root_y = 0
        self.on_resize_complete = None

    def candragged(self, b: bool):
        if b:
            self._candragged = self.widget.bind("<B1-Motion>", self.drag, add='+')
        else:
            if hasattr(self, "_candragged"):
                self.widget.unbind("<B1-Motion>", self._candragged)

    def _mouse_down(self, event):
        self.startx = event.x
        self.starty = event.y

    def _drag(self, event):
        try:
            self.place(x=self.winfo_x() + (event.x - self.startx), y=self.winfo_y() + (event.y - self.starty))
        except AttributeError:
            raise ValueError("The widget %s is not draggable" % widget)

    def set_on_resize_complete(self, on_resize_complete):
        self.on_resize_complete = on_resize_complete

    def _draw(self):
        self.delete("all")
        self.create_rectangle(-1, -1, -2, -2, tag='side', dash=4, outline=self._frame_border)
        self.tag_bind('side', "<Button-1>", self._mouse_down, add='+')
        self.tag_bind('side', "<B1-Motion>", self._drag, add='+')
        self.tag_bind('side', '<Enter>', lambda event: self.config(cursor='fleur'))
        self.tag_bind('side', '<Leave>', lambda event: self.config(cursor='arrow'))
        for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
            self.create_rectangle(-1, -1, -2, -2, tag=name, outline=self._frame_grip)
            self.tag_bind(name, "<Enter>", partial(self.on_mouse_enter, name))
            self.tag_bind(name, "<Leave>", partial(self.on_mouse_leave, name))
            self.tag_bind(name, "<Button-1>", partial(self.on_mouse_click, name))
            self.tag_bind(name, "<B1-Motion>", partial(self.on_mouse_move, name))
            self.tag_bind(name, "<ButtonRelease-1>", partial(self.on_mouse_release, name))

    def show(self, is_fill=False):
        width = self.winfo_width()
        height = self.winfo_height()
        self.coords('side', 6, 6, width - 6, height - 6)
        self.coords('nw', 0, 0, 7, 7)
        self.coords('sw', 0, height - 8, 7, height - 1)
        self.coords('w', 0, (height - 7) / 2, 7, (height - 7) / 2 + 7)
        self.coords('n', (width - 7) / 2, 0, (width - 7) / 2 + 7, 7)
        self.coords('s', (width - 7) / 2, height - 8, (width - 7) / 2 + 7, height - 1)
        self.coords('ne', width - 8, 0, width - 1, 7)
        self.coords('se', width - 8, height - 8, width - 1, height - 1)
        self.coords('e', width - 8, (height - 7) / 2, width - 1, (height - 7) / 2 + 7)
        if is_fill:
            for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
                self.itemconfig(name, fill='blue')

    def hide(self):
        self.coords('side', -1, -1, -2, -2, )
        for name in ('nw', 'w', 'sw', 'n', 's', 'ne', 'e', 'se'):
            self.coords(name, -1, -1, -2, -2)

    def on_mouse_enter(self, tag_name, event):
        if tag_name in ("nw", "sw", "ne", "se"):
            self["cursor"] = "sizing"
        elif tag_name in ("w", "e"):
            self["cursor"] = "sb_h_double_arrow"
        else:
            self["cursor"] = "sb_v_double_arrow"

    def on_mouse_leave(self, tag_name, event):
        if self.is_sizing:
            return
        self["cursor"] = "arrow"

    def on_mouse_click(self, tag_name, event):
        self.is_sizing = True
        self.start_x = event.x
        self.start_y = event.y
        self.start_root_x = event.x_root
        self.start_root_y = event.y_root
        self.old_width = self.winfo_width()
        self.old_height = self.winfo_height()
        self.old_pos_x = int(self.place_info()['x'])
        self.old_pos_y = int(self.place_info()['y'])

    def on_mouse_move(self, tag_name, event):
        if not self.is_sizing:
            return
        if 'e' in tag_name:
            width = max(0, self.old_width + (event.x - self.start_x))
            self.place_configure(width=width)
        if 'w' in tag_name:
            width = max(0, self.old_width + (self.start_root_x - event.x_root))
            to_x = event.x - self.start_x + int(self.place_info()['x'])
            self.place_configure(width=width, x=to_x)
        if 's' in tag_name:
            height = max(0, self.old_height + (event.y - self.start_y))
            self.place_configure(height=height)
        if 'n' in tag_name:
            height = max(0, self.old_height + (self.start_root_y - event.y_root))
            to_y = event.y - self.start_y + int(self.place_info()['y'])
            self.place_configure(height=height, y=to_y)
        self.after_idle(self.show)

    def on_mouse_release(self, tag_name, event):
        self.is_sizing = False
        if self.on_resize_complete is not None:
            self.on_resize_complete()
        self["cursor"] = "arrow"

    def mouse_down(self, event):
        self.widget.focus_set()
        self.__startx = event.x
        self.__starty = event.y

    def drag(self, event):
        self.place(x=self.winfo_x() + (event.x - self.__startx), y=self.winfo_y() + (event.y - self.__starty))

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "designer_frame": {
                    "border": "gray",
                    "grip": "gray"
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "designer_frame": {
                    "border": "gray",
                    "grip": "gray"
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "designer_frame" in dict:
                self.frame_grip = dict["designer_frame"]["grip"]
                self.frame_border = dict["designer_frame"]["border"]

            self._palette = dict

            self._frame_grip = self.frame_grip
            self._frame_border = self.frame_border

            try:
                self._draw()
            except AttributeError:
                pass
        else:
            return self._palette



from tkinter import Canvas
from .gradient import Gradient, Gradient2
from .roundrect import RoundRect
from .widget import AdwWidget


class AdwDrawEngine(Canvas, AdwWidget):
    def __init__(
            self,
            *args,
            highlightthickness=0,
            borderwidth=0,
            **kwargs
    ):
        super().__init__(*args, highlightthickness=highlightthickness, borderwidth=borderwidth, **kwargs)

        self.gradient = Gradient()
        self.gradient2 = Gradient2()

        self.roundrect = RoundRect()

    def gradient_draw(self, *args, **kwargs):
        """
        绘制渐变图形
        """
        self.gradient.draw(self, *args, **kwargs)

    def gradient2_draw(self, *args, **kwargs):
        """
        用渐变颜色填充画布
        """
        self.gradient2.draw1(self, *args, **kwargs)

    def gradient_recolor(self, *args, **kwargs):
        """
        修改渐变图形的颜色
        """
        self.gradient.recolor(self, *args, **kwargs)

    def gradient_redraw(self, *args, **kwargs):
        """
        重绘渐变图形
        """
        self.gradient.redraw(self, *args, **kwargs)

    def gradient_resize(self, *args, **kwargs):
        """
        修改渐变图形的位置大小
        """
        self.gradient.resize(self, *args, **kwargs)

    def roundrect_draw(self, *args, **kwargs):
        self.roundrect.draw(self, *args, **kwargs)

    def roundrect2_draw(self, *args, **kwargs):
        self.roundrect.draw2(self, *args, **kwargs)


from os import environ

win11 = {
    "light": {
        "window": {
            "back": "#f4f4f4"
        },

        "button": {
            "radius": 13,

            "default": {
                "back": "#fdfdfd",
                "border": "#ededed",
                "fore": "#202020",
                "border_width": 1,
            },

            "active": {
                "back": "#f9f9f9",
                "border": "#d5d5d5",
                "fore": "#202020",
                "border_width": 1,
            },

            "pressed": {
                "back": "#fafafa",
                "border": "#ebebeb",
                "fore": "#202020",
                "border_width": 1,
            },
        },

        "entry": {
            "radius": 13,

            "default": {
                "back": "#ffffff",
                "border": "#e6e6e6",
                "fore": "#000000",
                "border_width": 1,

                "bottomsheet": "#9c9c9c",
                "bottomsheet_width": 1,
            },

            "focus": {
                "back": "#ffffff",
                "border": "#ebebeb",
                "fore": "#000000",
                "border_width": 1,

                "bottomsheet": "#005fb8",
                "bottomsheet_width": 2,
            }
        },
    },
    "dark": {
        "window": {
            "back": "#202020"
        },

        "button": {
            "radius": 13,

            "default": {
                "back": "#2a2a2a",
                "border": "#313131",
                "fore": "#ebebeb",
                "border_width": 1,
            },

            "active": {
                "back": "#2f2f2f",
                "border": "#313131",
                "fore": "#ebebeb",
                "border_width": 1,
            },

            "pressed": {
                "back": "#232323",
                "border": "#2c2c2c",
                "fore": "#ebebeb",
                "border_width": 1,
            },
        },

        "entry": {
            "radius": 13,

            "default": {
                "back": "#2c2c2c",
                "border": "#383838",
                "fore": "#ffffff",
                "border_width": 1,

                "bottomsheet": "#686868",
                "bottomsheet_width": 1,
            },

            "focus": {
                "back": "#1c1c1c",
                "border": "#2c2c2c",
                "fore": "#ffffff",
                "border_width": 1,

                "bottomsheet": "#57c8ff",
                "bottomsheet_width": 2,
            }
        },
    }
}

from json import dumps

environ["ADWTHEME"] = dumps(win11)
environ["ADWTHEME.MODE"] = "light"


def theme(name=None):
    if name:
        if "ADWTHEME" in environ:
            from json import dumps
            environ["ADWTHEME"] = dumps(name)
    else:
        if "ADWTHEME" in environ:
            from json import loads
            return loads(environ["ADWTHEME"])


def theme_mode(mode=None):
    if mode:
        if "ADWTHEME.MODE" in environ:
            environ["ADWTHEME.MODE"] = mode
    else:
        if "ADWTHEME" in environ:
            return environ["ADWTHEME.MODE"]


class AdwThemed(object):
    def dark_palette(self):
        if theme():
            self.palette(theme()["dark"])

    def default_palette(self):
        if theme():
            self.palette(theme()[theme_mode()])

    def light_palette(self):
        if theme():
            if theme_mode():
                self.palette(theme()["light"])


from .button import AdwButton


class AdwTButton(AdwThemed, AdwButton):
    pass


from .entry import AdwEntry


class AdwTEntry(AdwThemed, AdwEntry):
    pass


from .window import AdwMainWindow


class AdwTMainWindow(AdwThemed, AdwMainWindow):
    def dark(self, enable: bool, width_icon=True):
        if width_icon:
            self.icon(enable)
        if enable:
            self.styles(["dark"])
        else:
            self.styles(["light"])

    深黑模式 = dark

    def theme(self, themename, thememode="system"):
        if themename == "win11":
            theme(win11)
        else:
            theme(themename)

        if thememode == "system":
            try:
                from darkdetect import isDark
            except ModuleNotFoundError:
                pass
            else:
                if isDark():
                    theme_mode("dark")
                else:
                    theme_mode("light")
        else:
            theme_mode(thememode)

        self.palette(theme()[theme_mode()])

        for child in self.winfo_children():
            if hasattr(child, "palette"):
                child.palette(theme()[theme_mode()])
                child.update()

    使用主题 = theme



from tkadwold.windows.theme import AdwTFrame

__all__ = [
    "AdwDesigner",
]


class AdwDesigner(AdwTFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.widgets = {}

        self.setup_project_panel()
        self.setup_widgets_panel()
        self.setup_designer_panel()

    def setup_project_panel(self):
        from tkadwold.windows.theme import AdwTListBox
        #from tkinterdnd2 import DND_ALL

        def delete(event):
            print(self.project_panel.select())
            if self.project_panel.select() is not None:
                _items = self.project_panel.cget("items")
                _items.remove(self.project_panel.select())
                self.widgets[self.project_panel.select()].destroy()
                self.project_panel.select(None)
                self.project_panel.update()

        def selection():
            try:
                self.after(100, self.widgets[self.project_panel.select()].widget.focus_set())
            except:
                pass

        self.project_panel = AdwTListBox(self.frame, width=130, item_selected=selection)
        self.project_panel.column(expand="no")
        self.project_panel.configure(item_selected=selection)
        self.project_panel.bind("<BackSpace>", delete)

    def setup_widgets_panel(self):
        from tkadwold.windows.theme import AdwTListBox
        #from tkinterdnd2 import DND_ALL

        self.widgets_panel = AdwTListBox(self.frame, items=["AdwTButton", "AdwTEntry", "AdwTText"], width=130)
        self.widgets_panel.bind("<Return>", lambda event: self.add_widget())

        self.widgets_panel.column(expand="no")

    def setup_designer_panel(self):
        from tkadwold.windows.theme import AdwTMDI
        self.designer_panel = AdwTMDI(self.frame)
        self.designer_window = self.designer_panel.create_designer_child(width=200, height=200)
        self.designer_panel.column()

    def add_widget(self):
        if self.widgets_panel.select() == "AdwTButton":
            from tkadwold.windows.theme import AdwTButton, AdwTDesignerFrame
            _ = AdwTDesignerFrame(self.designer_window[0], AdwTButton)
        elif self.widgets_panel.select() == "AdwTEntry":
            from tkadwold.windows.theme import AdwTEntry, AdwTDesignerFrame
            _ = AdwTDesignerFrame(self.designer_window[0], AdwTEntry)
        elif self.widgets_panel.select() == "AdwTText":
            from tkadwold.windows.theme import AdwTText, AdwTDesignerFrame
            _ = AdwTDesignerFrame(self.designer_window[0], AdwTText)
        _.place(x=20, y=40, width=100, height=40)
        _items = self.project_panel.cget("items")
        _items.append(_.winfo_id())
        self.project_panel.configure(items=_items)
        _.focus_set()
        self.project_panel.select(_.winfo_id())
        self.widgets[_.winfo_id()] = _


if __name__ == '__main__':
    from tkadwold import *
    root = Adwite(default_theme="win11")
    root.geometry("650x360")
    designer = AdwDesigner()
    designer.row()
    root.mainloop()


from tkadwold.windows.canvas.widget import AdwWidget


class AdwDrawBasicSeparator(AdwWidget):
    def __init__(self, *args, width=50, height=3, **kwargs):
        super().__init__(*args, width=width, height=height, highlightthickness=0, **kwargs)

        self.default_palette()

        self._draw(None)

    def _draw(self, evt=None):
        self.delete("all")

        self.separator = self.create_line(
            self.separator_border_width,
            self.winfo_height()/2,
            self.winfo_width()-self.separator_border_width,
            self.winfo_height() / 2,
            width=self.separator_border_width,
            fill=self.separator_back,
        )

        if self.separator_rounded:
            self.itemconfigure(self.separator, capstyle="round", joinstyle="round", smooth=True)

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "separator": {
                    "back": "gray15",
                    "border_width": 1,

                    "rounded": True
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "separator": {
                    "back": "gray85",
                    "border_width": 1,

                    "rounded": True
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "separator" in dict:
                self.separator_back = dict["separator"]["back"]
                self.separator_border_width = dict["separator"]["border_width"]

                self.separator_rounded = dict["separator"]["rounded"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwDrawSeparator(AdwDrawBasicSeparator):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkSeparator(AdwDrawBasicSeparator):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkadwold import Adw
    root = Adw()
    separator = AdwDrawSeparator()
    separator.pack(fill="both", expand="yes")
    separator = AdwDrawDarkSeparator()
    separator.pack(fill="both", expand="yes")
    root.mainloop()


from tkinter import Tk


class AdwRun(object):
    def __init__(self, root: Tk):
        self.root = root
        self.root.protocol("WM_DELETE_WINDOW", self.quit)

    def run(self):
        self.q = False
        while not self.q:
            self.root.update()

    def quit(self):
        self.q = True
        self.root.destroy()


def run(root):
    AdwRun(root).run()


if __name__ == '__main__':
    root = Tk()
    run(root)



from tkinter import Canvas


def rounded_rectangle(size, radius, fill):
    from PIL import Image, ImageDraw
    img = Image.new('RGBA', size, (0, 0, 0, 0))
    draw = ImageDraw.Draw(img)
    draw.rounded_rectangle((0, 0, size[0], size[1]), radius, fill)
    return img


poly = """
# poly.tcl

proc poly_round {win outline fill args} {
    if {[llength $args] % 3 != 0 || [llength $args] < 9} {
        error "wrong # args: should be \"poly_round\
                win outline fill x1 y1 d1 x2 y2 d2 x3 y3 d3 ?...?\""
    }

    # Determine the tag to use.
    if {![info exists ::poly_next_id]} {
        set ::poly_next_id 1
    }
    set tag poly#$::poly_next_id
    incr ::poly_next_id

    # Filter out illegal circles and collinear points.
    set pts [list]
    lassign [lrange $args 0 4] Ux Uy d Vx Vy
    foreach {d Wx Wy} [concat [lrange $args 5 end] [lrange $args 0 4]] {
        set test [expr {$Ux * ($Vy - $Wy) - $Vx * ($Uy - $Wy) +
                $Wx * ($Uy - $Vy)}]
        if {($d > 0) && $test != 0} {
            lappend pts $Vx $Vy $d $test
            lassign [list $Wx $Wy $Vx $Vy] Vx Vy Ux Uy
        } else {
            lassign [list $Wx $Wy] Vx Vy
        }
    }

    # V    C    T   W
    #  *---*----*-+-*-- Given: U, V, W, d
    #  |\ /    /|_|     Find:  S, E, T
    #  | *B   / |
    #  |/ \  /  |       The length of ES and ET each is d.
    # A*   \/   |
    #  |   /\   |       VB bisects angle UVW.  SE _|_ VU; TE _|_ VW.
    #  |  /  \  |       B is halfway between A and C.
    #  | /    \ |       Angles UVW and SET are not necessarily right.
    #  |/      \|       The length of AV and CV each is 1.
    # S*-+------*E
    #  |_|       \      The new polygon is along USTW.
    # U*          \     The new arc has center E, radius d, and angle SET, and
    #  |           \    it is tangential to VU at S and VW at T.

    # Calculate new polygon vertices and create arcs.
    set coords [list]
    lassign [lrange $pts 0 5] Ux Uy d test Vx Vy
    foreach {d test Wx Wy} [concat [lrange $pts 6 end] [lrange $pts 0 5]] {
        # Find A and C.
        foreach {pt x y} [list A $Ux $Uy C $Wx $Wy] {
            set      k [expr {sqrt(($Vx - $x) ** 2 + ($Vy - $y) ** 2)}]
            set ${pt}x [expr {($x - $Vx) / $k + $Vx}]
            set ${pt}y [expr {($y - $Vy) / $k + $Vy}]
        }

        # Find B.
        set Bx [expr {($Ax + $Cx) / 2.0}]
        set By [expr {($Ay + $Cy) / 2.0}]

        # Find the parameters for lines VB and VW.
        foreach {pt x y} [list B $Bx $By W $Wx $Wy] {
            set       k [expr {sqrt(($Vx - $x) ** 2 + ($Vy - $y) ** 2)}]
            set V${pt}a [expr {+($Vy - $y) / $k}]
            set V${pt}b [expr {-($Vx - $x) / $k}]
            set V${pt}c [expr {($Vx * $y - $Vy * $x) / $k}]
        }

        # Find point E.
        set sign [expr {$test < 0 ? -1 : +1}]
        set  k [expr {$VWa * $VBb - $VWb * $VBa}]
        set Ex [expr {(+$VWb * $VBc - ($VWc - $d * $sign) * $VBb) / $k}]
        set Ey [expr {(-$VWa * $VBc + ($VWc - $d * $sign) * $VBa) / $k}]

        # Find tangent points S and T.
        foreach {pt x y} [list S $Ux $Uy T $Wx $Wy] {
            set      k [expr {($Vx - $x) ** 2 + ($Vy - $y) ** 2}]
            set ${pt}x [expr {($Ex * ($Vx - $x) ** 2 + ($Vy - $y) *
                              ($Ey * ($Vx - $x) - $Vx * $y + $Vy * $x)) / $k}]
            set ${pt}y [expr {($Ex * ($Vx - $x) * ($Vy - $y) +
                              ($Ey * ($Vy - $y) ** 2 + ($Vx - $x) *
                              ($Vx * $y - $Vy * $x))) / $k}]
        }

        # Find directions for lines ES and ET.
        foreach {pt x y} [list S $Sx $Sy T $Tx $Ty] {
            set E${pt}d [expr {atan2($Ey - $y, $x - $Ex)}]
        }

        # Find start and extent directions.
        if {$ESd < 0 && $ETd > 0} {
            set start  [expr {180 / acos(-1) * $ETd}]
            set extent [expr {180 / acos(-1) * ($ESd - $ETd)}]
            if {$sign > 0} {
                set extent [expr {$extent + 360}]
            }
        } else {
            set start  [expr {180 / acos(-1) * $ESd}]
            set extent [expr {180 / acos(-1) * ($ETd - $ESd)}]
            if {$sign < 0 && $ESd > 0 && $ETd < 0} {
                set extent [expr {$extent + 360}]
            }
        }

        # Draw arc.
        set opts [list                             \
                [expr {$Ex - $d}] [expr {$Ey - $d}]\
                [expr {$Ex + $d}] [expr {$Ey + $d}]\
                -start $start -extent $extent]
        $win create arc {*}$opts -style pie -tags [list $tag pie]
        $win create arc {*}$opts -style arc -tags [list $tag arc]

        # Draw border line.
        if {[info exists prevx]} {
            $win create line $prevx $prevy $Sx $Sy -tags [list $tag line]
        } else {
            lassign [list $Sx $Sy] firstx firsty
        }
        lassign [list $Tx $Ty] prevx prevy

        # Remember coordinates for polygon.
        lappend coords $Sx $Sy $Tx $Ty

        # Rotate vertices.
        lassign [list $Wx $Wy $Vx $Vy] Vx Vy Ux Uy
    }

    # Draw final border line.
    $win create line $prevx $prevy $firstx $firsty -tags [list $tag line]

    # Draw fill polygon.
    $win create polygon {*}$coords -tags [list $tag poly]

    # Configure colors.
    $win itemconfigure $tag&&(poly||pie) -fill $fill
    $win itemconfigure $tag&&pie         -outline ""
    $win itemconfigure $tag&&line        -fill $outline -capstyle round
    $win itemconfigure $tag&&arc         -outline $outline

    # Set proper stacking order.
    $win raise $tag&&poly
    $win raise $tag&&pie
    $win raise $tag&&(line||arc)

    return $tag
}
       """

poly2 = """
 #----------------------------------------------------------------------
 #
 # RoundPoly -- Draw a polygon with rounded corners in the canvas, based
 # off of ideas and code from "Drawing rounded rectangles"
 #
 # Parameters:
 #       w - Path name of the canvas
 #       xy - list of coordinates of the vertices of the polygon
 #       radii - list of radius of the bend each each vertex
 #       args - Other args suitable to a 'polygon' item on the canvas
 #
 # Results:
 #       Returns the canvas item number of the rounded polygon.
 #
 # Side effects:
 #       Creates a rounded polygon in the canvas.
 #
 #----------------------------------------------------------------------
 
 proc RoundPoly {w xy radii args} {
    set lenXY [llength $xy]
    set lenR [llength $radii]
    if {$lenXY != 2 * $lenR} {
        error "wrong number of vertices and radii"
    }
 
    # Walk down vertices keeping previous, current and next
    lassign [lrange $xy end-1 end] x0 y0
    lassign $xy x1 y1
    eval lappend xy [lrange $xy 0 1]
    set knots {}                                ;# These are the control points
 
    for {set i 0} {$i < $lenXY} {incr i 2} {
        set radius [lindex $radii [expr {$i/2}]]
        set r [winfo pixels $w $radius]
 
        lassign [lrange $xy [expr {$i + 2}] [expr {$i + 3}]] x2 y2
        set z [_RoundPoly2 $x0 $y0 $x1 $y1 $x2 $y2 $r]
        eval lappend knots $z
 
        lassign [list $x1 $y1] x0 y0           ;# Current becomes previous
        lassign [list $x2 $y2] x1 y1           ;# Next becomes current
    }
    set n [eval $w create polygon $knots -smooth 1 $args]
    return $n
 }
 proc _RoundPoly2 {x0 y0 x1 y1 x2 y2 radius} {
    set d [expr { 2 * $radius }]
    set maxr 0.75
 
    set v1x [expr {$x0 - $x1}]
    set v1y [expr {$y0 - $y1}]
    set v2x [expr {$x2 - $x1}]
    set v2y [expr {$y2 - $y1}]
 
    set vlen1 [expr {sqrt($v1x*$v1x + $v1y*$v1y)}]
    set vlen2 [expr {sqrt($v2x*$v2x + $v2y*$v2y)}]
    if {$d > $maxr * $vlen1} {
        set d [expr {$maxr * $vlen1}]
    }
    if {$d > $maxr * $vlen2} {
        set d [expr {$maxr * $vlen2}]
    }
 
    lappend xy [expr {$x1 + $d * $v1x/$vlen1}] [expr {$y1 + $d * $v1y/$vlen1}]
    lappend xy $x1 $y1
    lappend xy [expr {$x1 + $d * $v2x/$vlen2}] [expr {$y1 + $d * $v2y/$vlen2}]
 
    return $xy
 }

"""

ARC = "arc"
CENTRAL = "central"
VERTICAL = "vertical"
HORIZONTAL = "horizontal"
CIRCULAR = "circular"


class AdwDrawEngine(Canvas):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def win32_high_dpi(self):
        """
        windows平台高DPI启用
        """
        from ctypes import windll
        windll.shcore.SetProcessDpiAwareness(2)

    def rgb_to_num(self, rgb):
        return int(rgb[1:3], 16), int(rgb[3:5], 16), int(rgb[5:], 16)

    def num_to_rgb(self, hexs: tuple):
        co = '#'
        for i in hexs:
            co += str(hex(i))[2:]
        return co

    def get_color_change(self, color1, color2, num=26):
        colors = []
        a1, a2, a3 = self.rgb_to_num(color1)
        b1, b2, b3 = self.rgb_to_num(color2)
        r, g, b = (b1 - a1), (b2 - a2), (b3 - a3)
        for i in range(num):
            t = i / num-1
            rgb = (int(a1 + r * t), int(a2 + g * t), int(a3 + b * t))
            colors.append(self.num_to_rgb(rgb))
        return colors

    def gradient_init(self):
        """
        初始化渐变引擎
        """
        self.tk.eval("""
        # Create color gradients of various shapes and sizes on a canvas.
#   Usage:
#       gradient draw    <canvas> <tags> <x> <y> <width> <height> <vertical|horizontal|circular|central|arc> <color1> <color2>  ?-animate? ?option? ...
#       gradient redraw  <canvas> <tags> <x> <y> <width> <height> <vertical|horizontal|circular|central|arc> <color1> <color2>  ?-animate? ?option? ...
#       gradient resize  <canvas> <tags> <x> <y> <width> <height>
#       gradient recolor <canvas> <tags> <color1> <color2>

namespace eval gradient {
    namespace ensemble create -subcommands {
        draw
        redraw
        recolor
        resize
        demo
    }

    # Create a gradient on a canvas. The gradient will be placed at the top of the display list.
    proc draw {canvas tags x y width height orient color1 color2 args} {
        # Ensure valid colors are provided while also converting to lists of 16-bit RGB values.
        if { [catch {winfo rgb . $color1} rgb1] } {
            error "invalid color: $color1"
        }
        if { [catch {winfo rgb . $color2} rgb2] } {
            error "invalid color: $color2"
        }
        
        # Delete the gradient if it already exists.
        $canvas delete [join $tags &&]
        
        # Create a hidden canvas item to store meta data about this gradient.
        set meta_data [list x $x  y $y  width $width  height $height  orient $orient  color1 $color1  color2 $color2  args $args]
        $canvas create line $x $y $x $y  -state hidden  -tags [list {*}$tags meta [list gradient: {*}$meta_data]]
        
        # Intercept the -animate option.
        if { "-animate" in $args} {
            set index [lsearch $args "-animate"]
            set args  [lreplace $args $index $index]
            set animate 1
        } else {
            set animate 0
        }
        
        # Draw the gradient.
        if { $orient eq "central" } {
            # Create the canvas rectangle items.
            foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                $canvas create rectangle $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  {*}$args
                if {$animate} {update}
            }
        } elseif { $orient eq "circular" } {
            # Create the canvas oval items.
            foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                $canvas create oval $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  {*}$args
                if {$animate} {update}
            }
        } elseif { $orient eq "arc" } {
            # Create the canvas arc items.
            foreach {x1 y1 x2 y2 color} [get_rects $x $y $width $height $rgb1 $rgb2] {
                $canvas create arc $x1 $y1 $x2 $y2  -outline $color  -width 2  -tags $tags  -style arc  {*}$args
                if {$animate} {update}
            }
        } elseif { $orient in [list "vertical" "horizontal"] } {        
            # Create the canvas line items.
            foreach {x1 y1 x2 y2 color} [get_lines $x $y $width $height $orient $rgb1 $rgb2] {
                $canvas create line $x1 $y1 $x2 $y2  -fill $color  -tags $tags  -capstyle projecting  {*}$args
                if {$animate} {update}
            }
        } else {
            # Delete the meta data and throw an error.
            $canvas delete [join $tags &&]
            error "invalid orientation: $orient; must be: vertical, horizontal, circular, central, or arc"
        }
        
        return
    }
    
    # Same as draw, but if the gradient already exists then it will retain its position in the canvas's display list after being redrawn.
    proc redraw {canvas tags x y width height orient color1 color2 args} {
        # If the gradient already exists, remember its location in the display list.
        set item_above [$canvas find above [join $tags &&]]
        
        # Draw the gradient.
        draw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
        
        # Move the gradient to its previous location in the display list, if any.
        if { $item_above ne "" } {
            $canvas lower [join $tags &&] $item_above
        }
        
        return
    }
    
    # Shortcut procedure to change the colors of the gradient. Same as redraw but introspectively determines some meta data. 
    # This will use the gradient's x/y location when it was last drawn.
    proc recolor {canvas tags color1 color2} {
        # Retrieve the gradient's meta data.
        set meta [get_meta_data $canvas $tags]
        set meta [dict remove $meta color1 color2]
        dict with meta {}
        
        # Redraw the gradient with the new colors.
        redraw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
        return     
    }
    
    # Shortcut procedure to resize and reposition a gradient. Same as redraw but introspectively determines some meta data. 
    proc resize {canvas tags x y width height} {
        # Retrieve the gradient's meta data.
        set meta [get_meta_data $canvas $tags]
        set meta [dict remove $meta x y width height]
        dict with meta {}
        
        # Recreate the canvas lines with the new dimensions.
        redraw $canvas $tags $x $y $width $height $orient $color1 $color2 {*}$args
        return
    }
    
    # Retrieve meta information on a gradient.
    proc get_meta_data {canvas tags} {
        if { [string is integer -strict $tags] } {
            # This is supposedly a canvas item ID instead of a list of tags describing a gradient. Avoid introducing subtle logic errors.
            error "cannot provide canvas item ID in lieu of gradient tags: $tags"
        }
        # This is a list of tags for a gradient. Find the hidden meta item for this gradient.
        set all_tags [$canvas gettags "meta&&[join $tags &&]"]
        set meta_data [lrange [lsearch -inline -index 0 $all_tags "gradient:"] 1 end]
        if { [llength $meta_data] == 0 } {
            error "cannot find meta data for gradient with tags: $tags"
        }
        return $meta_data
    }
    
    # For a canvas item ID, return all tags that are not gradient meta tags.
    proc get_non_meta_tags {canvas id} {
        set tags [$canvas gettags $id]
        set index [lsearch $tags "meta"]
        set tags [lreplace $tags $index $index]
        set index [lsearch -index 0 $tags "gradient:"]
        set tags [lreplace $tags $index $index]
        return $tags
    }

    # Calculate a list of center-to-outward colored rectangles suitable for [canvas create rectangle] or [canvas create oval].
    proc get_rects {x y width height rgb1 rgb2} {
        set rects [list]
        set x1 [expr {int($x)+1}]
        set y1 [expr {int($y)+1}]
        set x2 [expr {$x1+$width-1}]
        set y2 [expr {$y1+$height-1}]
        
        # Calculate each rectangle.
        if { $width > $height } {
            # Decrement X by one pixel in both directions. Decrement Y by a fraction in both directions.
            # The Y axis will be more heavily concentrated.
            set ratio  [expr { 1.0*$height/$width }]
            set length [expr { $width/2 }]
            while { $x2 > $x1 } {
                lappend rects $x1 $y1 $x2 $y2 [get_color $rgb1 $rgb2 $length [expr { ($x2-$x1)/2 }]]
                set x1 [expr { $x1+1 }]
                set x2 [expr { $x2-1 }]
                set y1 [expr { $y1+$ratio }]
                set y2 [expr { $y2-$ratio }]
            }
        } else {
            # Decrement Y by one pixel in both directions. Decrement X by a fraction in both directions.
            # The X axis will be more heavily concentrated (unless width==height).
            set ratio  [expr { 1.0*$width/$height }]
            set length [expr { $height/2 }]
            while { $y2 > $y1 } {
                lappend rects $x1 $y1 $x2 $y2 [get_color $rgb1 $rgb2 $length [expr { ($y2-$y1)/2 }]]
                set x1 [expr { $x1+$ratio }]
                set x2 [expr { $x2-$ratio }]
                set y1 [expr { $y1+1 }]
                set y2 [expr { $y2-1 }]
            }
        }
        return $rects
    }

    # Calculate a list of left-to-right or top-to-bottom colored lines suitable for [canvas create line].
    proc get_lines {x1 y1 width height orient rgb1 rgb2} {
        set lines [list]
        set x1 [expr {int($x1)}]
        set y1 [expr {int($y1)}]
        set x2 [expr {$x1+$width}]
        set y2 [expr {$y1+$height}]
        if { $orient eq "vertical" } {
            # Calculate the color for each horizontal line.
            for {set y $y1} {$y < $y2} {incr y} {
                lappend lines $x1 $y $x2 $y [get_color $rgb1 $rgb2 $height [expr {$y-$y1}]]
            }
        } elseif { $orient eq "horizontal" } {
            # Calculate the color for each vertical column.
            for {set x $x1} {$x < $x2} {incr x} {
                lappend lines $x $y1 $x $y2 [get_color $rgb1 $rgb2 $width [expr {$x-$x1}]]
            }
        } else {
            error "invalid orientation: $orient; must be: vertical or horizontal"
        }
        
        return $lines
    }

    # Calculates the color at the specified index between rgb1 and rgb2 where rgb1 and rgb2 are the specified length apart.
    proc get_color {rgb1 rgb2 length index} {
        # Throw an error if the index is out of bounds.
        if { $index < 0  ||  $index >= $length } {
            error "index $index is out of bounds for length $length"
        }
        
        lassign $rgb1 r1 g1 b1
        lassign $rgb2 r2 g2 b2
        
        # Determine the ratio between each starting component color and ending component color.
        set r_ratio [expr { 1.00*($r2-$r1+1)/$length }]
        set g_ratio [expr { 1.00*($g2-$g1+1)/$length }]
        set b_ratio [expr { 1.00*($b2-$b1+1)/$length }]

        # Calculate the new component colors at the given index. 
        set r [expr { int($r_ratio*$index+$r1) }]
        set g [expr { int($g_ratio*$index+$g1) }]
        set b [expr { int($b_ratio*$index+$b1) }]

        # A hacky workaround to make up for a lack of precision (or faulty math?).
        # The final pixel of the gradient should exactly match the color of rgb2
        if { $index == [expr {$length-1}] } {
            lassign $rgb2 r g b
        }

        # Convert the integer RGB values to a hex color.
        return [rgb_to_hex [list $r $g $b]]
    }
    
    # Convert a list of 16-bit RGB values to an 8-bit hex color. Using 8-bit hex colors instead of 16-bit
    # speeds up drawing of images two-fold (I haven't benchmarked this for canvases).
    proc rgb_to_hex {rgb} {
        lassign $rgb r g b
        set r [format %02x [expr {$r/256}]]
        set g [format %02x [expr {$g/256}]]
        set b [format %02x [expr {$b/256}]]
        return #$r$g$b
    }
    
    # Converts 8-bit RGB values to 16-bit RGB values.
    proc rgb_8_to_16_bit {rgb} {
        lassign $rgb r g b
        return [list [expr {$r*256}] [expr {$g*256}] [expr {$b*256}]]
    }
    
    # Returns a random 8-bit hex color.
    proc random_color {} {
        return [rgb_to_hex "[expr {int(rand()*65536)}] [expr {int(rand()*65536)}] [expr {int(rand()*65536)}]"]
    }

    # Redraw all gradients with random colors
    proc randomize_all_gradient_colors {canvas} {
        foreach id [$canvas find withtag meta] {
            set tags [get_non_meta_tags $canvas $id]
            recolor $canvas $tags [random_color] [random_color]
        }
      return
    }
    
    # Randomize colors for all canvas items.
    proc randomize_all_canvas_item_colors {canvas} {
        foreach id [$canvas find withtag all] {
            if { [$canvas type $id] in {window} } {
                continue
            }
            $canvas itemconfigure $id -fill [random_color]
            if { [$canvas type $id] in {rectangle arc oval} } {
                 $canvas itemconfigure $id -outline [random_color]
            }
        }
        return
    }
    
    # Run a demonstration.
    proc demo {} {
        # Create a toplevel window with a canvas filling the contents.
        set win .gradients
        if { [winfo exists $win] } {
            destroy $win
        }
        toplevel $win
        set cvs [canvas $win.cvs -highlightthickness 0]
        pack $cvs -fill both -expand yes
        wm geometry $win 800x600
        raise $win

        # Fill the bottom of the screen with a striped green fade.
        gradient draw $cvs footer 0 0 10 10 vertical white darkgreen -dash {3 1}
        bind $cvs <Configure> {
            gradient resize %W footer 0 [expr {%h-300}] %w 300
        }
        
        # Draw a strip of rainbow.
        set width  100
        set colors {white red orange yellow green blue purple violet white}
        for {set i 1} {$i < [llength $colors]} {incr i} {
            set c1 [lindex $colors $i-1]
            set c2 [lindex $colors $i]
            gradient draw $cvs "wave $i" [expr {$width*($i-1)}] 0 $width 50 horizontal $c1 $c2
        }
        
        # Draw some balls with diameters of 100.
        set colors {green red yellow blue orange white}
        for {set i 0} {$i < 6} {incr i} {
            gradient draw $cvs "beachball $i" 0 55 100 100 arc [lindex $colors $i] #333 -start [expr {$i*60}] -extent 60
        }
        set colors {cyan pink orange firebrick}
        for {set i 0} {$i < [llength $colors]} {incr i} {
            gradient draw $cvs "quadball $i" 110 55 100 100 arc [lindex $colors $i] #eee -start [expr {$i*90}] -extent 90
        }
        gradient draw $cvs ball1    220 55 100 100 circular #bbbbbb white
        gradient draw $cvs ball2    330 55 100 100 circular #888 #eee
        gradient draw $cvs ball3    440 55 100 100 circular black grey
        gradient draw $cvs ball4    550 55 100 100 circular black #555
        gradient draw $cvs ball5    660 55 100 100 circular cyan black
        
        # Draw some boxes.
        gradient draw $cvs box1   0 160 150 100 horizontal white firebrick
        gradient draw $cvs box2 150 160 200 100 central    black firebrick
        gradient draw $cvs box3 350 160 100 100 horizontal firebrick black
        gradient draw $cvs box4 450 160 200 100 central    firebrick black
        gradient draw $cvs box5 650 160 150 100 horizontal black white
        
        # Display some control buttons.
        frame $cvs.f
        button $cvs.f.b1 -text "Redraw with random gradient colors" -command [list [namespace current]::randomize_all_gradient_colors $cvs]
        button $cvs.f.b2 -text "Relaunch demo with default colors" -command [list [namespace current]::demo]
        button $cvs.f.b3 -text "Chaos" -command [list [namespace current]::randomize_all_canvas_item_colors $cvs]
        pack $cvs.f.b1 $cvs.f.b2 $cvs.f.b3 -fill x
        $cvs create window 400 400 -window $cvs.f

        return
    }
}
        """)

    def gradient_demo(self):
        """
        启动渐变引擎示例
        """
        self.gradient_init()
        self.tk.call("gradient", "demo")

    def gradient_resize(self, tags: str, x: int, y: int, width: int, height: int):
        """
        修改渐变图形的大小位置
        """
        self.gradient_init()
        self.tk.call("gradient", "resize", self._w, tags, x, y, width, height)

    def gradient_recolor(self, tags: str, x: int, y: int, color1, color2):
        """
        修改渐变组件的颜色
        """
        self.gradient_init()
        self.tk.call("gradient", "color", self._w, tags, color1, color2)

    def gradient_draw(self, tags: str, x: int, y: int, width: int, height: int, orient, color1, color2, *args,
                      **kwargs):
        """
        绘制渐变图形
        """
        self.gradient_init()
        id = self.tk.call("gradient", "draw", self._w, tags, x, y, width, height, orient, color1, color2)
        self.itemconfigure(tags, *args, **kwargs)
        return id

    def gradient_redraw(self, *args, **kwargs):
        """
        同gradient_draw
        """
        self.gradient_draw(*args, **kwargs)

    def create_gradient_v_rectangle(self, x, y, width, height, tags, color1, color2, *args, **kwargs):
        """
        创建水平渐变矩形

        :param x: 图形x位置
        :param y: 图形y位置
        :param width: 图形宽度
        :param height: 图形高度
        :param tags: 标签名
        :param color1: 渐变颜色1
        :param color2: 渐变颜色2
        :return:
        """
        return self.gradient_draw(tags=tags, x=x, y=y, width=width, height=height, color1=color1, color2=color2,
                                  orient=VERTICAL, *args, **kwargs)

    def create_gradient_h_rectangle(self, x, y, width, height, tags, color1, color2, *args, **kwargs):
        """
        创建垂直渐变矩形

        :param x: 图形x位置
        :param y: 图形y位置
        :param width: 图形宽度
        :param height: 图形高度
        :param tags: 标签名
        :param color1: 渐变颜色1
        :param color2: 渐变颜色2
        :return:
        """
        return self.gradient_draw(tags=tags, x=x, y=y, width=width, height=height, color1=color1, color2=color2,
                                  orient=HORIZONTAL, *args, **kwargs)

    def create_gradient_circular(self, x, y, width, height, tags, color1, color2, *args, **kwargs):
        """
        创建垂直渐变圆形

        :param x: 图形x位置
        :param y: 图形y位置
        :param width: 图形宽度
        :param height: 图形高度
        :param tags: 标签名
        :param color1: 渐变颜色1
        :param color2: 渐变颜色2
        :return:
        """
        return self.gradient_draw(tags=tags, x=x, y=y, width=width, height=height, color1=color1, color2=color2,
                                  orient=CIRCULAR, *args, **kwargs)

    def draw_gradient(self, color1, color2, type="x"):
        self.tk.eval("""
  proc + {n1 n2} {
    expr {$n1 + $n2}
  }
  proc - {n1 n2} {
    expr {$n1 - $n2}
  }
  proc * {n1 n2} {
    expr {$n1 * $n2}
  }
  proc / {n1 n2} {
    expr {$n1 / $n2}
  }
  proc toInt {n} {
    expr int($n)
  }
  
  proc drawGradient {win type col1Str col2Str} {
    $win delete gradient
    
    set width [winfo width $win]
    set height [winfo height $win]
    
    lassign [winfo rgb $win $col1Str] r1 g1 b1
    lassign  [winfo rgb $win $col2Str] r2 g2 b2
    set rRange [- $r2.0 $r1]
    set gRange [- $g2.0 $g1]
    set bRange [- $b2.0 $b1]
  
    if {$type == "x"} {
      set rRatio [/ $rRange $width]
      set gRatio [/ $gRange $width]
      set bRatio [/ $bRange $width]
    
      for {set x 0} {$x < $width} {incr x} {
        set nR [toInt [+ $r1 [* $rRatio $x]]]
        set nG [toInt [+ $g1 [* $gRatio $x]]]
        set nB [toInt [+ $b1 [* $bRatio $x]]]
  
        set col [format {%4.4x} $nR]
        append col [format {%4.4x} $nG]
        append col [format {%4.4x} $nB]
        $win create line $x 0 $x $height -tags gradient -fill #${col}
      }
    } else {
      set rRatio [/ $rRange $height]
      set gRatio [/ $gRange $height]
      set bRatio [/ $bRange $height]
  
      for {set y 0} {$y < $height} {incr y} {
        set nR [toInt [+ $r1 [* $rRatio $y]]]
        set nG [toInt [+ $g1 [* $gRatio $y]]]
        set nB [toInt [+ $b1 [* $bRatio $y]]]
  
        set col [format {%4.4x} $nR]
        append col [format {%4.4x} $nG]
        append col [format {%4.4x} $nB]
        $win create line 0 $y $width $y -tags gradient -fill #${col}
      }
    }
    return $win
  }
        """)

        self.tk.call("drawGradient", self._w, type, color1, color2)

    def create_round_rectangle(self, x1, y1, x2, y2, radius: float = 5, width=2, fill="white", outline="black"):
        """
        创建圆角矩形 ： 自绘版
        """
        # 自研
        _radius = radius * 2
        nw = self.create_arc(x1, y1, x1 + _radius, y1 + _radius, start=90, extent=90, width=width, fill=fill,
                             outline=outline)  # ⌜ north | west
        sw = self.create_arc(x1, y2, x1 + _radius, y2 - _radius, start=180, extent=90, width=width, fill=fill,
                             outline=outline)  # ⌞ south | wast
        ne = self.create_arc(x2 - _radius, y1, x2, y1 + _radius, start=0, extent=90, width=width, fill=fill,
                             outline=outline)  # ⌝ north | east
        se = self.create_arc(x2 - _radius, y2, x2, y2 - _radius, start=270, extent=90, width=width, fill=fill,
                             outline=outline)  # ⌟ south | east

        w = self.create_line(x1, y1 + _radius / 2, x1, y2 - _radius / 2, width=width, fill=outline)  # | left
        n = self.create_line(x1 + _radius / 2, y1, x2 - _radius / 2, y1, width=width, fill=outline)  # —— up
        e = self.create_line(x2, y1 + _radius / 2, x2, y2 - _radius / 2, width=width, fill=outline)  # | right
        s = self.create_line(x1 + _radius / 2, y2, x2 - _radius / 2, y2, width=width, fill=outline)  # —— down

        top = self.create_rectangle(x1 + _radius / 2 - width, y1 + width / 2, x2 - _radius / 2 + width,
                                    y1 + _radius / 2, fill=fill,
                                    width=0)  # ▭ top

        center = self.create_rectangle(x1 + width / 2, y1 + _radius / 2 - width, x2 - width / 2,
                                       y2 - _radius / 2 + width, fill=fill,
                                       width=0)  # ▭ center

        bottom = self.create_rectangle(x1 + _radius / 2 - width, y2 - _radius / 2, x2 - _radius / 2 + width,
                                       y2 - width / 2, fill=fill,
                                       width=0)  # ▭ bottom

        return {
            "nw": nw, "sw": sw, "ne": ne, "se": se, "w": w, "n": n, "e": e, "s": s,
            "top": top, "center": center, "bottom": bottom
        }

    create_round_rect = create_round_rectangle

    def create_round_rectangle2(self, x0, y0, x3, y3, radius, *args, **kwargs):
        """
        创建圆角矩形 ： 其他开发者制作 ： 边框有问题（不推荐）
        """
        # wiki上
        self.tk.eval("""
#----------------------------------------------------------------------
#
# roundRect --
#
#       Draw a rounded rectangle in the canvas.
#
# Parameters:
#       w - Path name of the canvas
#       x0, y0 - Co-ordinates of the upper left corner, in pixels
#       x3, y3 - Co-ordinates of the lower right corner, in pixels
#       radius - Radius of the bend at the corners, in any form
#                acceptable to Tk_GetPixels
#       args - Other args suitable to a 'polygon' item on the canvas
#
# Results:
#       Returns the canvas item number of the rounded rectangle.
#
# Side effects:
#       Creates a rounded rectangle as a smooth polygon in the canvas.
#
#----------------------------------------------------------------------

proc roundRect { w x0 y0 x3 y3 radius args } {

set r [winfo pixels $w $radius]
set d [expr { 2 * $r }]

# Make sure that the radius of the curve is less than 3/8
# size of the box!

set maxr 0.75

if { $d > $maxr * ( $x3 - $x0 ) } {
    set d [expr { $maxr * ( $x3 - $x0 ) }]
}
if { $d > $maxr * ( $y3 - $y0 ) } {
    set d [expr { $maxr * ( $y3 - $y0 ) }]
}

set x1 [expr { $x0 + $d }]
set x2 [expr { $x3 - $d }]
set y1 [expr { $y0 + $d }]
set y2 [expr { $y3 - $d }]

set cmd [list $w create polygon]
lappend cmd $x0 $y0
lappend cmd $x1 $y0
lappend cmd $x2 $y0
lappend cmd $x3 $y0
lappend cmd $x3 $y1
lappend cmd $x3 $y2
lappend cmd $x3 $y3
lappend cmd $x2 $y3
lappend cmd $x1 $y3
lappend cmd $x0 $y3
lappend cmd $x0 $y2
lappend cmd $x0 $y1
lappend cmd -smooth 1
return [eval $cmd $args]
}
        """)
        rect = self.tk.call("roundRect", self._w, x0, y0, x3, y3, radius)
        self.itemconfig(rect, *args, **kwargs)
        return rect

    create_round_rect2 = create_round_rectangle2

    def create_round_rectangle3(self, tag, x, y, width, height, radius, fill: str = "black", outline: str = "black",
                                *args, **kwargs):
        """
        创建圆角矩形 ： 其他开发者制作 ： 圆角看起来更舒服
        """
        # wiki上 圆角看起来更舒服
        self.tk.eval("""
proc roundRect2 {w L T Rad width height fill outline tag} {

  $w create oval $L $T [expr $L + $Rad] [expr $T + $Rad] -fill $fill -outline $outline -tag $tag
  $w create oval [expr $width-$Rad] $T $width [expr $T + $Rad] -fill $fill -outline $outline -tag $tag
  $w create oval $L [expr $height-$Rad] [expr $L+$Rad] $height -fill $fill -outline $outline -tag $tag
  $w create oval [expr $width-$Rad] [expr $height-$Rad] [expr $width] $height -fill $fill -outline $outline -tag $tag
  $w create rectangle [expr $L + ($Rad/2.0)] $T [expr $width-($Rad/2.0)] $height -fill $fill -outline $outline -tag $tag
  $w create rectangle $L [expr $T + ($Rad/2.0)] $width [expr $height-($Rad/2.0)] -fill $fill -outline $outline -tag $tag
}
            """)
        _rect = self.tk.call("roundRect2", self._w, x, y, radius, width, height, fill, outline, tag)
        self.itemconfig(_rect, *args, **kwargs)
        return _rect

    create_round_rect3 = create_round_rectangle3

    def create_round_rectangle4(self, x1, y1, x2, y2, radius, **kwargs):
        """
        创建圆角矩形 ： 其他开发者制作
        """
        points = [x1 + radius, y1,
                  x1 + radius, y1,
                  x2 - radius, y1,
                  x2 - radius, y1,
                  x2, y1,
                  x2, y1 + radius,
                  x2, y1 + radius,
                  x2, y2 - radius,
                  x2, y2 - radius,
                  x2, y2,
                  x2 - radius, y2,
                  x2 - radius, y2,
                  x1 + radius, y2,
                  x1 + radius, y2,
                  x1, y2,
                  x1, y2 - radius,
                  x1, y2 - radius,
                  x1, y1 + radius,
                  x1, y1 + radius,
                  x1, y1]

        _poly = self.create_polygon(points, **kwargs, smooth=True)

        return _poly

    create_round_rect4 = create_round_rectangle4

    def polygon_round(self, win, outline="black", fill="black", *args, **kwargs):
        self.tk.eval(poly)
        _poly = self.tk.eval(f"poly_round {win} {outline} {fill}")
        self.itemconfig(_poly, *args, **kwargs)
        return _poly

    poly_round = polygon_round

    def demo_polygon_round(self):
        demo = """
 package require Tcl 8.5
 package require Tk

 proc draw {win} {
     global demo

     set sharp_pts [list]
     set round_pts [list]
     for {set id 0} {$id < $demo(num_pts)} {incr id} {
         set x [expr {([lindex [$win coords vtx#$id] 0] +
                       [lindex [$win coords vtx#$id] 2]) / 2}]
         set y [expr {([lindex [$win coords vtx#$id] 1] +
                       [lindex [$win coords vtx#$id] 3]) / 2}]
         lappend sharp_pts $x $y
         lappend round_pts $x $y $demo(radius)
     }

     .c delete sharp_poly
     .c create polygon {*}$sharp_pts -outline gray50 -fill ""\
             -dash {6 5} -tags {sharp_poly}

     if {[info exists demo(tag)]} {
         .c delete $demo(tag)
     }
     set demo(tag) [poly_round .c $demo(outline) $demo(fill) {*}$round_pts]
     .c itemconfigure $demo(tag) -width $demo(thickness)

     .c raise vtx
 }

 proc down {win x y} {
     global demo

     $win dtag selected
     $win addtag selected withtag current
     $win raise current
     set demo(last_x) $x
     set demo(last_y) $y
 }
 
 proc move {win x y} {
     global demo

     if {[info exists demo(last_x)]} {
         $win move selected\
                 [expr {$x - $demo(last_x)}]\
                 [expr {$y - $demo(last_y)}]
         set demo(last_x) $x
         set demo(last_y) $y

         draw $win
     }
 }

 proc main {args} {
     global demo

     array set demo {
         num_pts 3       radius 20      thickness 1
         outline black   fill   white   background gray
         width   400     height 400
     }
     foreach {option value} $args {
         set option [regsub {^-} $option ""]
         if {![info exists demo($option)]} {
             puts "Options: -[join [array names demo] " -"]"
             exit
         } else {
             set demo([regsub {^-} $option ""]) $value
         }
     }

     canvas .c -width $demo(width) -height $demo(height) -highlightthickness 0\
             -background $demo(background)
     pack .c
     wm title . "Round Polygon Demo"
     wm resizable . 0 0

     set 2pi [expr {2 * acos(-1)}]
     set cx [expr {$demo(width)  / 2}]; set sx [expr {$demo(width)  * 3 / 8}]
     set cy [expr {$demo(height) / 2}]; set sy [expr {$demo(height) * 3 / 8}]
     for {set id 0} {$id < $demo(num_pts)} {incr id} {
         set x [expr {$cx + $sx * cos(($id + 0.5) * $2pi / $demo(num_pts))}]
         set y [expr {$cy - $sy * sin(($id + 0.5) * $2pi / $demo(num_pts))}]
         .c create oval [expr {$x - 3}] [expr {$y - 3}]\
                        [expr {$x + 3}] [expr {$y + 3}]\
                        -tags [list vtx vtx#$id] -fill brown
     }

     .c bind vtx <Any-Enter> {.c itemconfigure current -fill red}
     .c bind vtx <Any-Leave> {.c itemconfigure current -fill brown}
     .c bind vtx <ButtonPress-1> {down .c %x %y}
     .c bind vtx <ButtonRelease-1> {.c dtag selected}
     bind .c <B1-Motion> {move .c %x %y}

     focus .c
     draw .c
 }

 main

        """

        self.tk.eval(poly)
        self.tk.eval(demo)

    def create_svg_image(self, x, y, content: str, **kwargs):
        from tempfile import mkstemp
        from tksvg import SvgImage

        _, file = mkstemp(suffix=".svg")
        print(file)
        with open(file, "w") as f:
            f.write(content)
            f.close()
        image = SvgImage(file=file)

        i = self.create_image(x, y, image=image, **kwargs)

        return i

    def draw_copy_icon(self, __x, __y, size=10, padding=10, radius=18):
        _1 = self.create_round_rect4(__x, __y, __x + size * 5, __y + size * 5, radius)
        _2 = self.create_round_rect4(__x + padding, __y + padding, __x + size * 5 - padding, __y + size * 5 - padding,
                                     radius / 2, fill="white")
        _3 = self.create_round_rect4(__x + size * 2, __y + size * 2, __x + size * 7, __y + size * 7, radius)
        _4 = self.create_round_rect4(__x + size * 2 + padding, __y + size * 2 + padding, __x + size * 7 - padding,
                                     __y + size * 7 - padding, radius / 2, fill="white")
        return _1, _2, _3, _4

    def create_round_rectangle5(self, __x, __y, __width, __height, fill="black", radius=16, *args, **kwargs):
        from PIL import ImageTk
        img = rounded_rectangle((__width, __height), radius, fill, *args, **kwargs)
        photo = ImageTk.PhotoImage(img)

        return self.create_image(__x, __y, image=photo, anchor='nw')


if __name__ == '__main__':
    from tkinter import Tk

    root = Tk()

    canvas = AdwDrawEngine()

    # canvas.create_round_rect3(10, 15, 15, 150, 150, 50)
    # canvas.gradient_demo()
    # canvas.bind("<Configure>", lambda event: canvas.draw_gradient("#87e9bb", "#d3a6f5", "x"))

    canvas.gradient_demo()
    canvas.pack(fill="both", expand="yes")

    root.mainloop()



from tkadwold.windows.widgets.adw import Adw, run as adw_run, AdwRun
from tkadwold.windows.widgets.toplevel import AdwToplevel
from tkadwold.windows.widgets.label import AdwLabel, AdwDarkLabel
from tkadwold.windows.widgets.mdi import AdwMDI
from tkadwold.windows.widgets.listbox import AdwListBox, AdwRoundListBox2, AdwRoundListBox3
from tkadwold.windows.widgets.stack import AdwStack



# 0.2.0加入
from tkadwold.windows.canvas.drawengine import AdwDrawEngine, HORIZONTAL, VERTICAL, ARC, CIRCULAR, CENTRAL
from tkadwold.windows.canvas.button import AdwDrawButton, AdwDrawDarkButton, AdwDrawAccentButton, \
    AdwDrawRoundButton, AdwDrawRoundDarkButton, AdwDrawRoundAccentButton, \
    AdwDrawRoundButton2, AdwDrawRoundDarkButton2, AdwDrawRoundAccentButton2, \
    AdwDrawRoundButton3, AdwDrawRoundDarkButton3, AdwDrawRoundAccentButton3, \
    AdwDrawCircularButton, AdwDrawCircularDarkButton
from tkadwold.windows.canvas.entry import AdwDrawEntry, AdwDrawDarkEntry, AdwDrawRoundEntry, AdwDrawRoundDarkEntry, \
    AdwDrawRoundEntry3, AdwDrawRoundDarkEntry3
from tkadwold.windows.canvas.textbox import AdwDrawText, AdwDrawDarkText, AdwDrawRoundText, AdwDrawRoundDarkText, \
    AdwDrawRoundText3, AdwDrawRoundDarkText3
from tkadwold.windows.canvas.frame import AdwDrawFrame, AdwDrawDarkFrame, AdwDrawRoundFrame, AdwDrawDarkRoundFrame, \
    AdwDrawRoundFrame3, AdwDrawDarkRoundFrame3

# 0.3加入
from tkadwold.windows.canvas.widget import AdwWidget
# from tkadwold.canvas.label import AdwDrawLabel, AdwDrawDarkLabel 已移除



from tkadwold.windows.canvas.widget import AdwWidget
from tkinter import Frame


# Frame
class AdwDrawBasicFrame(AdwWidget):
    def __init__(self, *args, width=200, height=200, **kwargs):
        super().__init__(*args, width=width, height=height, bd=0, highlightthickness=0, **kwargs)

        self.default_palette()

        self.frame = Frame(self)

        self._draw(None)

    def _draw(self, evt):
        self.delete("all")

        # 绘制框架边框
        self.frame_frame = self.create_rectangle(
            0, 0, self.winfo_width(), self.winfo_height(),
            width=self.frame_border_width,
            outline=self.frame_border, fill=self.frame_back,
        )

        self.frame.configure(background=self.frame_back, bd=0, )

        # 绘制框架区域
        self.frame_f = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            window=self.frame,
            width=self.winfo_width() - 6 - self.frame_border_width - self.frame_padding,
            height=self.winfo_height() - 6 - self.frame_border_width - self.frame_padding,
        )

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "frame": {
                    "back": "#ffffff",
                    "border": "#eaeaea",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "frame": {
                    "back": "#0f0f0f",
                    "border": "#333333",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "frame" in dict:
                self.frame_back = dict["frame"]["back"]
                self.frame_border = dict["frame"]["border"]
                self.frame_border_width = dict["frame"]["border_width"]
                self.frame_padding = dict["frame"]["padding"]

            self._palette = dict

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwDrawFrame(AdwDrawBasicFrame):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkFrame(AdwDrawBasicFrame):
    def default_palette(self):
        self.palette_dark()


class AdwDrawBasicRoundFrame(AdwDrawBasicFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        try:
            self.configure(background=self.master.cget("background"))
        except:
            pass

    def _draw(self, evt):
        self.delete("all")

        # 绘制框架边框
        self.frame_frame = self.create_round_rect2(
            self.frame_border_width, self.frame_border_width,
            self.winfo_width() - self.frame_border_width, self.winfo_height() - self.frame_border_width,
            self.frame_radius,
            width=self.frame_border_width,
            outline=self.frame_border, fill=self.frame_back,
        )

        self.frame.configure(background=self.frame_back, bd=0, )

        # 绘制框架区域
        self.frame_f = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            window=self.frame,
            width=self.winfo_width() - 6 - self.frame_border_width - self.frame_padding,
            height=self.winfo_height() - 6 - self.frame_border_width - self.frame_padding,
        )

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "frame": {
                    "radius": 6,
                    "back": "#0f0f0f",
                    "border": "#333333",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "frame": {
                    "radius": 6,
                    "back": "#0f0f0f",
                    "border": "#333333",
                    "border_width": 2,
                    "padding": 0
                }
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if dict is not None:
            if "frame" in dict:
                self.frame_radius = dict["frame"]["radius"]
        if dict is None:
            return self._palette


class AdwDrawRoundFrame(AdwDrawBasicRoundFrame):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkRoundFrame(AdwDrawBasicRoundFrame):
    def default_palette(self):
        self.palette_dark()


class AdwDrawBasicRoundFrame3(AdwDrawBasicRoundFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        try:
            self.configure(background=self.master.cget("background"))
        except:
            pass

    def _draw(self, evt):
        self.delete("all")

        # 绘制框架边框
        self.create_round_rect4(
            self.frame_border_width, self.frame_border_width,
            self.winfo_width() - self.frame_border_width,
            self.winfo_height() - self.frame_border_width,
            self.frame_radius,
            width=self.frame_border_width,
            outline=self.frame_border, fill=self.frame_back,
        )

        self.entry_frame = "button_frame"

        self.frame.configure(background=self.frame_back, bd=0, )

        # 绘制框架区域
        self.frame_f = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            window=self.frame,
            width=self.winfo_width() - 6 - self.frame_border_width - self.frame_padding,
            height=self.winfo_height() - 6 - self.frame_border_width - self.frame_padding,
        )

    def default_palette(self):
        self.palette_light()


class AdwDrawRoundFrame3(AdwDrawBasicRoundFrame3):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkRoundFrame3(AdwDrawBasicRoundFrame3):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkinter import Tk

    root = Tk()

    frame = AdwDrawDarkFrame()
    frame.pack(fill="both", expand="yes", )

    frame2 = AdwDrawDarkFrame(frame.frame)
    frame2.pack(fill="both", expand="yes", padx=8, pady=8)

    frame3 = AdwDrawDarkRoundFrame()
    frame3.pack(fill="both", expand="yes", )

    frame4 = AdwDrawDarkRoundFrame(frame3.frame)
    frame4.pack(fill="both", expand="yes", padx=8, pady=8)

    frame5 = AdwDrawDarkRoundFrame3()
    frame5.pack(fill="both", expand="yes", )

    frame6 = AdwDrawDarkRoundFrame3(frame5.frame)
    frame6.pack(fill="both", expand="yes", padx=8, pady=8)

    root.mainloop()



from tkadwold import *
from tkinter.font import nametofont, families, Font

index = """
tkAdw文档
*********

文档 > 主页

* 00 | 应用程序接口

  * 按钮

  * 绘画引擎

  * 条目输入框

  * 容器

  * 标签

  * 多文档窗口

  * 分割线

  * 文本输入框

  * 小组件

  * 主题组件

* 01 | 简介

  * tkAdw是什么

  * 我为什么要做tkAdw

  * 其他类似扩展库

* 02 | 主题组件

  * 主题原理

  * 设置主题

  * 使用主题组件

  * 版本警告

"""

p00 = """
00 | 应用程序接口
*****************

文档 > 应用程序接口 : sphinx.ext.autodoc

======================================================================


按钮
====

class tkadwold.windows.canvas.button.AdwDrawAccentButton(*args, width=120, height=40, text: str = '', command=None, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawBasicButton(*args, width=120, height=40, text: str = '', command=None, **kwargs)

   自绘基础按钮

   configure(**kwargs)

      Configure resources of a widget.

      The values for resources are specified as keyword arguments. To
      get an overview about the allowed keyword arguments call the
      method keys.

   update() -> None

      多执行一道绘制

class tkadwold.windows.canvas.button.AdwDrawBasicCircularButton(*args, width=120, height=120, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawBasicRoundButton(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawButton(*args, width=120, height=40, text: str = '', command=None, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawCircularButton(*args, width=120, height=120, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawCircularDarkButton(*args, width=120, height=120, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawDarkButton(*args, width=120, height=40, text: str = '', command=None, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundAccentButton(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundAccentButton2(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundAccentButton3(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundButton(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundButton2(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundButton3(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundDarkButton(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundDarkButton2(*args, **kwargs)

class tkadwold.windows.canvas.button.AdwDrawRoundDarkButton3(*args, **kwargs)


绘画引擎
========

class tkadwold.windows.canvas.drawengine.AdwDrawEngine(*args, **kwargs)

   create_gradient_circular(x, y, width, height, tags, color1, color2, *args, **kwargs)

      创建垂直渐变圆形

      参数:
         * **x** – 图形x位置

         * **y** – 图形y位置

         * **width** – 图形宽度

         * **height** – 图形高度

         * **tags** – 标签名

         * **color1** – 渐变颜色1

         * **color2** – 渐变颜色2

      返回:
   create_gradient_h_rectangle(x, y, width, height, tags, color1, color2, *args, **kwargs)

      创建垂直渐变矩形

      参数:
         * **x** – 图形x位置

         * **y** – 图形y位置

         * **width** – 图形宽度

         * **height** – 图形高度

         * **tags** – 标签名

         * **color1** – 渐变颜色1

         * **color2** – 渐变颜色2

      返回:
   create_gradient_v_rectangle(x, y, width, height, tags, color1, color2, *args, **kwargs)

      创建水平渐变矩形

      参数:
         * **x** – 图形x位置

         * **y** – 图形y位置

         * **width** – 图形宽度

         * **height** – 图形高度

         * **tags** – 标签名

         * **color1** – 渐变颜色1

         * **color2** – 渐变颜色2

      返回:
   create_round_rect(x1, y1, x2, y2, radius: float = 5, width=2, fill='white', outline='black')

      创建圆角矩形 ： 自绘版

   create_round_rect2(x0, y0, x3, y3, radius, *args, **kwargs)

      创建圆角矩形 ： 其他开发者制作 ： 边框有问题（不推荐）

   create_round_rect3(tag, x, y, width, height, radius, fill: str = 'black', outline: str = 'black', *args, **kwargs)

      创建圆角矩形 ： 其他开发者制作 ： 圆角看起来更舒服

   create_round_rect4(x1, y1, x2, y2, radius, **kwargs)

      创建圆角矩形 ： 其他开发者制作

   create_round_rectangle(x1, y1, x2, y2, radius: float = 5, width=2, fill='white', outline='black')

      创建圆角矩形 ： 自绘版

   create_round_rectangle2(x0, y0, x3, y3, radius, *args, **kwargs)

      创建圆角矩形 ： 其他开发者制作 ： 边框有问题（不推荐）

   create_round_rectangle3(tag, x, y, width, height, radius, fill: str = 'black', outline: str = 'black', *args, **kwargs)

      创建圆角矩形 ： 其他开发者制作 ： 圆角看起来更舒服

   create_round_rectangle4(x1, y1, x2, y2, radius, **kwargs)

      创建圆角矩形 ： 其他开发者制作

   gradient_demo()

      启动渐变引擎示例

   gradient_draw(tags: str, x: int, y: int, width: int, height: int, orient, color1, color2, *args, **kwargs)

      绘制渐变图形

   gradient_init()

      初始化渐变引擎

   gradient_recolor(tags: str, x: int, y: int, color1, color2)

      修改渐变组件的颜色

   gradient_redraw(*args, **kwargs)

      同gradient_draw

   gradient_resize(tags: str, x: int, y: int, width: int, height: int)

      修改渐变图形的大小位置

   win32_high_dpi()

      windows平台高DPI启用


条目输入框
==========

class tkadwold.windows.canvas.entry.AdwDrawBasicEntry(*args, width=120, height=40, animated=5, text: str = '', show: str = None, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawBasicRoundEntry(*args, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawBasicRoundEntry3(*args, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawDarkEntry(*args, width=120, height=40, animated=5, text: str = '', show: str = None, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawEntry(*args, width=120, height=40, animated=5, text: str = '', show: str = None, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawRoundDarkEntry(*args, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawRoundDarkEntry3(*args, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawRoundEntry(*args, **kwargs)

class tkadwold.windows.canvas.entry.AdwDrawRoundEntry3(*args, **kwargs)


容器
====

class tkadwold.windows.canvas.frame.AdwDrawBasicFrame(*args, width=200, height=200, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawBasicRoundFrame(*args, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawBasicRoundFrame3(*args, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawDarkFrame(*args, width=200, height=200, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawDarkRoundFrame(*args, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawDarkRoundFrame3(*args, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawFrame(*args, width=200, height=200, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawRoundFrame(*args, **kwargs)

class tkadwold.windows.canvas.frame.AdwDrawRoundFrame3(*args, **kwargs)


标签
====

class tkadwold.windows.widgets.label.AdwBasicLabel(*args, text: str = '', **kwargs)

class tkadwold.windows.widgets.label.AdwDarkLabel(*args, text: str = '', **kwargs)

class tkadwold.windows.widgets.label.AdwLabel(*args, text: str = '', **kwargs)


多文档窗口
==========

class tkadwold.windows.widgets.mdi.AdwMDI(*args, **kwargs)

   多文档窗口

   create_child()

      创建一个子窗口

      返回:
         AdwTFrame


分割线
======

class tkadwold.windows.canvas.separator.AdwDrawBasicSeparator(*args, width=50, height=3, **kwargs)

class tkadwold.windows.canvas.separator.AdwDrawDarkSeparator(*args, width=50, height=3, **kwargs)

class tkadwold.windows.canvas.separator.AdwDrawSeparator(*args, width=50, height=3, **kwargs)


文本输入框
==========

class tkadwold.windows.canvas.textbox.AdwDrawBasicRoundText(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawBasicRoundText3(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawBasicText(*args, text: str = '', width=120, height=80, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawDarkText(*args, text: str = '', width=120, height=80, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawRoundDarkText(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawRoundDarkText3(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawRoundText(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawRoundText3(*args, **kwargs)

class tkadwold.windows.canvas.textbox.AdwDrawText(*args, text: str = '', width=120, height=80, **kwargs)


小组件
======

class tkadwold.windows.canvas.widget.AdwWidget(*args, **kwargs)

   基础绘制组件类

   特性：自动将背景颜色设为父组件背景颜色


主题组件
========

class tkadwold.windows.theme.AdwTButton(*args, **kwargs)

class tkadwold.windows.theme.AdwTEntry(*args, **kwargs)

class tkadwold.windows.theme.AdwTFrame(*args, **kwargs)

class tkadwold.windows.theme.AdwTLabel(*args, text: str = '', **kwargs)

class tkadwold.windows.theme.AdwTSeparator(*args, width=50, height=3, **kwargs)

class tkadwold.windows.theme.AdwTText(*args, **kwargs)

class tkadwold.windows.theme.Adwite(*args, title: str = 'adw', icon='light', windark: bool = True, wincaption=None, **kwargs)

"""

p01 = """
01 | 简介
*********

文档 > 简介

======================================================================


tkAdw是什么
===========

tkAdw是"tkinter"的扩展组件库，运用"tkinter.Canvas"组件绘制高级、现代化
的界面。将每个组件都设为"tkinter.Canvas"，在上面绘制组件，当大小位置发
生改变便重新绘制。


我为什么要做tkAdw
=================

之前我都在用"customtkinter"做界面，但是用着用着我就发现，有些样式无法
绘制出来，如"winui3"中的"TextBox(Entry)"界面效果下划线无法实现，所以这
才想到可以自己做一个界面框架。


其他类似扩展库
==============

* "customtkinter" ： 它和"tkadwold"一样使用"tkinter.Canvas"绘制组件，也同
  样支持圆角绘制；不同的是他们采用的是"macos"平台组件设计的，而"tkadwold"
  是采用"gtk"和"winui3"的样式进行设计的，还有就是按钮的点击事件不同，
  他们采用的是 "鼠标碰到并按下按钮" 触发事件，而"tkadwold"这是更加高级复
  制的"鼠标碰到并按下按钮直到鼠标放开并碰到鼠标"触发事件。

* "tinui" ： *待添加*

======================================================================

编写：XiangQinxi

"""

p02 = """
02 | 主题组件
*************

文档 > 主题组件

======================================================================


主题原理
========

将主题加入环境变量"tkAdwite.DefaultTheme"，需要时对环境变量进行读取。


设置主题
========

root需设置为"tkadwold.windows.theme.Adwite"，"set_default_theme"设置主题
及附加深浅模式设置，有三个内置主题"win11" "gtk" "bilibili" "metro", 我
为内置主题都设计了深浅两种主题，可以在附加参数填入"system" "dark"
"light"（"system"参数需安装"darkdetect"，如果未安装且设置为"system"将
自动设为"light"主题）。


使用主题组件
============

暂有这些组件"AdwTButton" "AdwTEntry" "AdwTFrame" "AdwTLabel"
"AdwTSeparator" "AdwTText"。需配合"Adwite"设置主题

   from tkadwold import Adwite, AdwTButton

   root = Adwite()
   root.set_default_theme("win11", "dark")  # 设置主题，第一个参数为`主题设置`或`内置主题名`，第二个参数为`内置主题深浅模式`

   button = AdwTButton(text="Hello World")
   button.pack(fill="x", padx=15, pady=15)

   root.mainloop()


版本警告
========

"主题组件"自"0.3.0"版本才加入，请注意"tkadwold"的版本号

======================================================================

编写：XiangQinxi

"""

root = Adwite()
root.set_default_theme("metro")
root.geometry("980x520")

page = 1
print(families())
read = AdwTText(root)
font = nametofont("TkDefaultFont")
font.configure(size=11)
read.font(font)
read.tdelete("1.0", "end")
read.tinsert("1.0", index)
read.pack(fill="both", expand="yes", padx=15, pady=15)


def up():
    global page
    if page == 1:
        page = 2
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p00)
    elif page == 2:
        page = 3
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p01)
    elif page == 3:
        page = 4
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p02)
    elif page == 4:
        page = 1
        read.tdelete("1.0", "end")
        read.tinsert("1.0", index)


def down():
    global page
    if page == 1:
        page = 4
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p02)
    elif page == 2:
        page = 1
        read.tdelete("1.0", "end")
        read.tinsert("1.0", index)
    elif page == 3:
        page = 2
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p00)
    elif page == 4:
        page = 3
        read.tdelete("1.0", "end")
        read.tinsert("1.0", p01)


combar = AdwTFrame(root, height=40)
combar.frame_border_width = 0
combar.frame_radius = 0

comdowm = AdwTButton(combar.frame, text="上一页", command=down)
comdowm.pack(fill="both", expand="yes", side="left", padx=5, pady=5)

comup = AdwTButton(combar.frame, text="下一页", command=up)
comup.pack(fill="both", expand="yes", side="right", padx=5, pady=5)

combar.pack(fill="x", side="bottom", ipady=10)

root.mainloop()


from tkadwold.windows.theme import Adwite, AdwTFrame

from pygubu.api.v1 import (
    BuilderObject,
    register_widget,
    register_custom_property,
)
from pygubu.i18n import _


_designer_tab_label = _("TkAdwite")
_plugin_uid = "tkadwold"


class AdwiteBuilder(BuilderObject):
    class_ = Adwite
    container = True


class AdwTFrameBuilder(BuilderObject):
    class_ = AdwTFrame

    container = True


register_widget(
    f'{_plugin_uid}.AdwTFrame', AdwTFrameBuilder,
    'AdwTFrame',
    ('ttk', _designer_tab_label)
)




from tkinter.font import Font, nametofont
from tkadwold.windows.canvas.widget import AdwWidget


# Entry
class AdwDrawBasicEntry(AdwWidget):
    def __init__(self, *args, width=120, height=40, animated=5, text: str = "",
                 show: str = None, **kwargs):

        super().__init__(*args, width=width, height=height, highlightthickness=0, **kwargs)

        from tkinter import StringVar, Entry

        self.var = StringVar()
        self.var.set(text)

        self.entry_text_font = nametofont("TkDefaultFont")

        self.entry = Entry(
            self, bd=0, textvariable=self.var, highlightthickness=0, font=self.entry_text_font,
            show=show
        )

        self.default_palette()

        self.text: str = text

        self._entry_back = self.entry_back
        self._entry_border = self.entry_border
        self._entry_text_back = self.entry_text_back
        self._entry_border_width = self.entry_border_width
        self._entry_bottom_line = self.entry_bottom_line
        self._entry_bottom_width = self.entry_bottom_width

        self.bind("<Button>", self._click, add="+")
        self.bind("<Enter>", self._hover, add="+")
        self.bind("<Leave>", self._hover_release, add="+")
        # self.bind("<FocusIn>", self._focus, add="+")
        self.entry.bind("<FocusIn>", self._focus, add="+")
        # self.bind("<FocusOut>", self._focusout, add="+")
        self.entry.bind("<FocusOut>", self._focusout, add="+")

        self._draw(None)

    def set(self, text: str):
        self.var.set(text)

    def get(self):
        return self.var.get()

    def _draw(self, evt=None):
        self.delete("all")

        self.entry.configure(font=self.entry_text_font)

        # 绘制输入框边框
        self.entry_frame = self.create_rectangle(
            0, 0, self.winfo_width() - 1, self.winfo_height() - 1,
            width=self._entry_border_width,
            outline=self._entry_border, fill=self._entry_back,
        )

        # 绘制输入框输入区域
        self.entry_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self._entry_border_width - 5 - self.entry_padding[0],
            height=self.winfo_height() - self._entry_border_width - 5 - self.entry_padding[1],
            window=self.entry
        )

        # 绘制输入框美化效果
        from _tkinter import TclError
        try:
            self.entry_bottom = self.create_rectangle(self._entry_border_width,
                                                      self.winfo_height() - self._entry_bottom_width - self._entry_border_width,
                                                      self.winfo_width() - self._entry_border_width,
                                                      self.winfo_height() - self._entry_border_width,
                                                      fill=self._entry_bottom_line, outline=self._entry_bottom_line,
                                                      width=0)
        except TclError:
            pass

        if self._entry_bottom_width == 0:
            self.delete(self.entry_bottom)

        self.tag_raise(self.entry_bottom, self.entry_text)

        self.entry.configure(background=self._entry_back, foreground=self._entry_text_back,
                             insertbackground=self._entry_text_back)

    def _focus(self, evt=None):
        self._entry_back = self.entry_focusin_back
        self._entry_border = self.entry_focusin_border
        self._entry_border_width = self.entry_focusin_border_width
        self._entry_text_back = self.entry_focusin_text_back

        self._entry_bottom_line = self.entry_focusin_bottom_line
        self._entry_bottom_width = self.entry_focusin_bottom_width

        self._draw(None)

    def _focusout(self, evt=None):
        self._entry_back = self.entry_back
        self._entry_border = self.entry_border
        self._entry_border_width = self.entry_border_width
        self._entry_text_back = self.entry_text_back
        self._entry_bottom_line = self.entry_bottom_line
        self._entry_bottom_width = self.entry_bottom_width

        self._draw(None)

    def _click(self, evt=None):
        self.focus_set()

    def _hover(self, evt=None):
        self.hover = True

    def _hover_release(self, evt=None):
        if not self.focus_get():
            self.hover = False
            self._entry_back = self.entry_back
            self._entry_border = self.entry_border
            self._entry_border_width = self.entry_border_width
            self._entry_text_back = self.entry_text_back
            self._entry_bottom_line = self.entry_bottom_line
            self._entry_bottom_width = self.entry_bottom_width

            self._draw(None)

    def font(self, font: Font = None):
        if font is None:
            return self.entry_text_font
        else:
            self.entry_text_font = font

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "entry": {
                    "padding": (3, 4),

                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#5f5f5f",
                    "border_width": 1,

                    "bottom_line": "#eaeaea",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#1a1a1a",
                        "border_width": 1,

                        "bottom_line": "#185fb4",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "entry": {
                    "padding": (3, 4),

                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#cecece",
                    "border_width": 1,

                    "bottom_line": "#ffffff",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#2f2f2f",
                        "border": "#383838",
                        "text_back": "#ffffff",
                        "border_width": 1,

                        "bottom_line": "#4cc2ff",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette(self, dict=None):
        if dict is not None:
            if "entry" in dict:
                self.entry_padding = dict["entry"]["padding"]

                self.entry_back = dict["entry"]["back"]
                self.entry_border = dict["entry"]["border"]
                self.entry_text_back = dict["entry"]["text_back"]
                self.entry_border_width = dict["entry"]["border_width"]

                self.entry_bottom_line = dict["entry"]["bottom_line"]
                self.entry_bottom_width = dict["entry"]["bottom_width"]

                if "focusin" in dict["entry"]:
                    self.entry_focusin_back = dict["entry"]["focusin"]["back"]
                    self.entry_focusin_border = dict["entry"]["focusin"]["border"]
                    self.entry_focusin_text_back = dict["entry"]["focusin"]["text_back"]
                    self.entry_focusin_border_width = dict["entry"]["focusin"]["border_width"]

                    self.entry_focusin_bottom_line = dict["entry"]["focusin"]["bottom_line"]
                    self.entry_focusin_bottom_width = dict["entry"]["focusin"]["bottom_width"]

            self._palette = dict

            self._entry_back = self.entry_back
            self._entry_border = self.entry_border
            self._entry_text_back = self.entry_text_back
            self._entry_border_width = self.entry_border_width
            self._entry_bottom_line = self.entry_bottom_line
            self._entry_bottom_width = self.entry_bottom_width

            try:
                self._draw(None)
            except AttributeError:
                pass
        else:
            return self._palette


class AdwDrawEntry(AdwDrawBasicEntry):
    def default_palette(self):
        self.palette_light()


class AdwDrawDarkEntry(AdwDrawBasicEntry):
    def default_palette(self):
        self.palette_dark()


# Rounded Entry
class AdwDrawBasicRoundEntry(AdwDrawBasicEntry):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _other(self):
        if hasattr(self, "button_frame_back"):
            self.configure(background=self.button_frame_back, borderwidth=0)

    def border_radius(self, radius=None):
        if radius is None:
            return self.button_radius
        else:
            self.button_radius = radius

    def _draw(self, evt):
        self.delete("all")

        # 绘制输入框边框
        self.entry_frame = self.create_round_rect2(
            2, 2, self.winfo_width() - 3, self.winfo_height() - 3, self.entry_radius,
            width=self._entry_border_width,
            outline=self._entry_border, fill=self._entry_back,
        )

        # 绘制输入框输入区
        self.entry_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self._entry_border_width - 5 - self.entry_padding[0],
            height=self.winfo_height() - self._entry_border_width - 5 - self.entry_padding[1],
            window=self.entry
        )

        # 绘制输入框美化效果
        self.entry_bottom = self.create_rectangle(3 + self.entry_radius / 2,
                                                  self.winfo_height() - self._entry_bottom_width - 3,
                                                  self.winfo_width() - 3 - self.entry_radius / 2,
                                                  self.winfo_height() - 3.5,
                                                  fill=self._entry_bottom_line, outline=self._entry_bottom_line,
                                                  width=0)

        if self._entry_bottom_width == 0:
            self.delete(self.entry_bottom)

        self.tag_raise(self.entry_bottom, self.entry_text)

        self.entry.configure(background=self._entry_back, foreground=self._entry_text_back,
                             insertbackground=self._entry_text_back)

    def default_palette(self):
        self.palette_light()

    def palette_light(self):
        self.palette(
            {
                "entry": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#fdfdfd",
                    "border": "#eaeaea",
                    "text_back": "#5f5f5f",
                    "border_width": 1,

                    "bottom_line": "#eaeaea",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#f9f9f9",
                        "border": "#e2e2e2",
                        "text_back": "#1a1a1a",
                        "border_width": 1,

                        "bottom_line": "#185fb4",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette_dark(self):
        self.palette(
            {
                "entry": {
                    "radius": 6,
                    "padding": (3, 4),

                    "back": "#353535",
                    "border": "#454545",
                    "text_back": "#cecece",
                    "border_width": 1,

                    "bottom_line": "#ffffff",
                    "bottom_width": 0,

                    "focusin": {
                        "back": "#2f2f2f",
                        "border": "#454545",
                        "text_back": "#ffffff",
                        "border_width": 1,

                        "bottom_line": "#4cc2ff",
                        "bottom_width": 2,
                    }
                },
            }
        )

    def palette(self, dict=None):
        super().palette(dict)
        if "entry" in dict:
            self.entry_radius = dict["entry"]["radius"]


class AdwDrawRoundEntry(AdwDrawBasicRoundEntry):
    def default_palette(self):
        self.palette_light()


class AdwDrawRoundDarkEntry(AdwDrawBasicRoundEntry):
    def default_palette(self):
        self.palette_dark()


class AdwDrawBasicRoundEntry3(AdwDrawBasicRoundEntry):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _other(self):
        self.configure(background=self.master.cget("bg"), borderwidth=0)

    def border_radius(self, radius=None):
        if radius is None:
            return self.button_radius
        else:
            self.button_radius = radius

    def _draw(self, evt):
        self.delete("all")

        # 绘制输入框边框
        self.create_round_rect4(
            0,
            0,
            self.winfo_width() - 1,
            self.winfo_height() - 1,
            self.entry_radius,
            width=self._entry_border_width,
            outline=self._entry_border, fill=self._entry_back,
        )

        self.entry_frame = "button_frame"

        # 绘制输入框输入区域
        self.entry_text = self.create_window(
            self.winfo_width() / 2, self.winfo_height() / 2,
            width=self.winfo_width() - self._entry_border_width - self.entry_padding[0] - 2,
            height=self.winfo_height() - self._entry_border_width - self.entry_padding[1] - 2,
            window=self.entry
        )

        # 绘制输入框美化效果
        self.entry_bottom = self.create_rectangle(1 + self.entry_radius / 5,
                                                  self.winfo_height() - self._entry_bottom_width - 1,
                                                  self.winfo_width() - 1 - self.entry_radius / 5,
                                                  self.winfo_height() - 1,
                                                  fill=self._entry_bottom_line, outline=self._entry_bottom_line,
                                                  width=0)

        if self._entry_bottom_width == 0:
            self.delete(self.entry_bottom)

        self.tag_raise(self.entry_bottom, self.entry_text)

        self.entry.configure(background=self._entry_back, foreground=self._entry_text_back,
                             insertbackground=self._entry_text_back)

    def default_palette(self):
        self.palette_light()


class AdwDrawRoundEntry3(AdwDrawBasicRoundEntry3):
    def default_palette(self):
        self.palette_light()


class AdwDrawRoundDarkEntry3(AdwDrawBasicRoundEntry3):
    def default_palette(self):
        self.palette_dark()


if __name__ == '__main__':
    from tkinter import Tk

    root = Tk()

    entry1 = AdwDrawEntry(text="Hello")
    entry1.pack(fill="x", padx=5, pady=5)

    entry2 = AdwDrawDarkEntry(text="Hello")
    entry2.pack(fill="x", padx=5, pady=5)

    entry3 = AdwDrawRoundEntry(text="Hello")
    entry3.pack(fill="x", padx=5, pady=5)

    entry4 = AdwDrawRoundDarkEntry(text="Hello")
    entry4.pack(fill="x", padx=5, pady=5)

    entry5 = AdwDrawRoundEntry3(text="Hello")
    entry5.pack(fill="x", padx=5, pady=5)

    entry6 = AdwDrawRoundDarkEntry3(text="Hello")
    entry6.pack(fill="x", padx=5, pady=5)

    entry_pwd1 = AdwDrawEntry(text="Hello", show=">")
    entry_pwd1.pack(fill="x", padx=5, pady=5)

    root.mainloop()
